//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file  sdi.h
* @brief SDcard low level driver functions
*
* Copyright (C) 2007 Parrot S.A.
*
* @author     florent.bayendrian@parrot.fr
* @date       2006-02-08
*******************************************************************************
*/

#ifndef _SDI_H
#define _SDI_H

#include <cyg/infra/cyg_ass.h>
#include <cyg/infra/diag.h>             // diag_printf
#include <cyg/hal/s3c2412x.h>

/*
 * Description:
 * This macro refers to the Nth register of block at __B
 */
#define SDI_REGISTER(__N)   (* ((volatile cyg_uint32 *) (SDI_State.Base) + __N))

/*
 * Description:
 * Build a mask for the specified bits
 *
 * Implementation:
 * __bws - a width/shift pair.
 *
 * Returns  a mask with the bits to be addressed set and all others cleared
 *
 */
#define BIT_MASK(__bws) ((cyg_uint32)(((bw ## __bws)==32)?0xFFFFFFFF:((1U << (bw ## __bws)) - 1)) << (bs ## __bws))


/*
 * Description:
 * Access the specified bits from a word of data as an integer value
 *
 * Implementation:
 * __datum - the word of data to be accessed
 * __bws - a width/shift pair.
 *
 * Returns  The relevant bits masked from the data word shifted to bit zero
 *
 */
#define BIT_GET(__datum, __bws) ((cyg_uint32)(((__datum) & ((cyg_uint32) BIT_MASK(__bws))) >> (bs ## __bws)))


/*
 * Description:
 * Place the specified value into the specified bits of a word of data
 *
 * Implementation:
 * __datum - the word of data to be accessed
 * __bws - a width/shift pair.
 * __val - the data value to be shifted into the specified bits.
 *
 */
#define BIT_SET(__datum, __bws, __val) \
    ((__datum) = ((cyg_uint32) (__datum) & (cyg_uint32)~(BIT_MASK(__bws))) | \
    ((cyg_uint32) ((cyg_uint32)(__val) << (cyg_uint32)(bs ## __bws)) & (BIT_MASK(__bws))))


/*
 * Description:
 * The register offsets for the SDI controller registers are defined
 * below:
 */
#define SDI_REG_SDICON               ((cyg_uint32)  0)
#define SDI_REG_SDIPRE               ((cyg_uint32)  1)
#define SDI_REG_SDICMDARG            ((cyg_uint32)  2)
#define SDI_REG_SDICMDCON            ((cyg_uint32)  3)
#define SDI_REG_SDICMDSTA            ((cyg_uint32)  4)
#define SDI_REG_SDIRSP0              ((cyg_uint32)  5)
#define SDI_REG_SDIRSP1              ((cyg_uint32)  6)
#define SDI_REG_SDIRSP2              ((cyg_uint32)  7)
#define SDI_REG_SDIRSP3              ((cyg_uint32)  8)
#define SDI_REG_SDIDTIMER            ((cyg_uint32)  9)
#define SDI_REG_SDIBSIZE             ((cyg_uint32) 10)
#define SDI_REG_SDIDATCON            ((cyg_uint32) 11)
#define SDI_REG_SDIDATCNT            ((cyg_uint32) 12)
#define SDI_REG_SDIDATSTA            ((cyg_uint32) 13)
#define SDI_REG_SDIFSTA              ((cyg_uint32) 14)
#define SDI_REG_SDIINTMSK            ((cyg_uint32) 15)
#define SDI_REG_SDIDAT               ((cyg_uint32) 16)


/*
 * Description:
 * Bit offsets and masks for SDI Control Register (SDICON)
 */
#define bsSDI_SDICON_ENCLK        ((cyg_uint32) 0)
#define bwSDI_SDICON_ENCLK        ((cyg_uint32) 1)
#define bsSDI_SDICON_RWAITEN      ((cyg_uint32) 2)
#define bwSDI_SDICON_RWAITEN      ((cyg_uint32) 1)
#define bsSDI_SDICON_RCVIOINT    ((cyg_uint32) 3)
#define bwSDI_SDICON_RCVIOINT    ((cyg_uint32) 1)
#define bsSDI_SDICON_BYTEORDER   ((cyg_uint32) 4)
#define bwSDI_SDICON_BYTEORDER   ((cyg_uint32) 1)
#define bsSDI_SDICON_CTYP        ((cyg_uint32) 5)
#define bwSDI_SDICON_CTYP        ((cyg_uint32) 1)
#define bsSDI_SDICON_HOLDMGN     ((cyg_uint32) 6)
#define bwSDI_SDICON_HOLDMGN     ((cyg_uint32) 2)
#define bsSDI_SDICON_SDRESET     ((cyg_uint32) 8)
#define bwSDI_SDICON_SDRESET     ((cyg_uint32) 1)

/*
 * Description:
 * Bit offsets and masks for SDI Baud Rate Prescaler Register (SDIPRE)
 */
#define bsSDI_SDIPRE_VALUE        ((cyg_uint32) 0)
#define bwSDI_SDIPRE_VALUE        ((cyg_uint32) 8)

/*
 * Description:
 * Bit offsets and masks for SDI Command Control Register (SDICMDCON)
 */
#define bsSDI_SDICMDCON_CMDINDEX  ((cyg_uint32)  0)
#define bwSDI_SDICMDCON_CMDINDEX  ((cyg_uint32)  8)
#define bsSDI_SDICMDCON_CMST      ((cyg_uint32)  8)
#define bwSDI_SDICMDCON_CMST      ((cyg_uint32)  1)
#define bsSDI_SDICMDCON_WAITRSP   ((cyg_uint32)  9)
#define bwSDI_SDICMDCON_WAITRSP   ((cyg_uint32)  1)
#define bsSDI_SDICMDCON_LONGRSP   ((cyg_uint32) 10)
#define bwSDI_SDICMDCON_LONGRSP   ((cyg_uint32)  1)
#define bsSDI_SDICMDCON_WITHDATA  ((cyg_uint32) 11)
#define bwSDI_SDICMDCON_WITHDATA  ((cyg_uint32)  1)
#define bsSDI_SDICMDCON_ABORTCMD  ((cyg_uint32) 12)
#define bwSDI_SDICMDCON_ABORTCMD  ((cyg_uint32)  1)
#define SDI_SDICMDCON_SENDERHOST      ((cyg_uint32) (1<<6))


/*
 * Description:
 * Bit offsets and masks for SDI Command Status Register (SDICMDSTA)
 */
#define bsSDI_SDICMDSTA_RSPINDEX  ((cyg_uint32)  0)
#define bwSDI_SDICMDSTA_RSPINDEX  ((cyg_uint32)  8)

#define SDI_SDICMDSTA_CMDON       ((cyg_uint32) (1<< 8))
#define SDI_SDICMDSTA_RSPFIN      ((cyg_uint32) (1<< 9))
#define SDI_SDICMDSTA_CMDTOUT     ((cyg_uint32) (1<<10))
#define SDI_SDICMDSTA_CMDSENT     ((cyg_uint32) (1<<11))
#define SDI_SDICMDSTA_RESPCRC     ((cyg_uint32) (1<<12))

#define SDI_SDICMDSTA_EVENT \
        (SDI_SDICMDSTA_RSPFIN  | \
         SDI_SDICMDSTA_CMDTOUT | \
         SDI_SDICMDSTA_CMDSENT | \
         SDI_SDICMDSTA_RESPCRC)

/*
 * Description:
 * Bit offsets and masks for SDI Response Register 1 (SDIRSP1)
 */
#define bsSDI_SDIRSP1_RESPONSE1  ((cyg_uint32)  0)
#define bwSDI_SDIRSP1_RESPONSE1  ((cyg_uint32)  24)
#define bsSDI_SDIRSP1_RCRC7      ((cyg_uint32)  24)
#define bwSDI_SDIRSP1_RCRC7      ((cyg_uint32)  8)


/*
 * Description:
 * Bit offsets and masks for SDI Data / Busy Timer Register (SDIDTIMER)
 */
#define bsSDI_SDIDTIMER_DATATIMER  ((cyg_uint32)  0)
#define bwSDI_SDIDTIMER_DATATIMER  ((cyg_uint32)  23)


/*
 * Description:
 * Bit offsets and masks for SDI Block Size Register (SDIBSIZE)
 */
#define bsSDI_SDIBSIZE_BLKSIZE  ((cyg_uint32)  0)
#define bwSDI_SDIBSIZE_BLKSIZE  ((cyg_uint32)  12)


/*
 * Description:
 * Bit offsets and masks for SDI Data Control Register (SDIDATCON)
 */
#define bsSDI_SDIDATCON_BLKNUM    ((cyg_uint32)  0)
#define bwSDI_SDIDATCON_BLKNUM    ((cyg_uint32) 12)
#define bsSDI_SDIDATCON_DATMODE   ((cyg_uint32) 12)
#define bwSDI_SDIDATCON_DATMODE   ((cyg_uint32)  2)
#define bsSDI_SDIDATCON_DTST      ((cyg_uint32) 14)
#define bwSDI_SDIDATCON_DTST      ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_ENDMA     ((cyg_uint32) 15)
#define bwSDI_SDIDATCON_ENDMA     ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_WIDEBUS   ((cyg_uint32) 16)
#define bwSDI_SDIDATCON_WIDEBUS   ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_BLKMODE   ((cyg_uint32) 17)
#define bwSDI_SDIDATCON_BLKMODE   ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_BACMD     ((cyg_uint32) 18)
#define bwSDI_SDIDATCON_BACMD     ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_RACMD     ((cyg_uint32) 19)
#define bwSDI_SDIDATCON_RACMD     ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_TARSP     ((cyg_uint32) 20)
#define bwSDI_SDIDATCON_TARSP     ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_PRDTYPE   ((cyg_uint32) 21)
#define bwSDI_SDIDATCON_PRDTYPE   ((cyg_uint32)  1)
#define bsSDI_SDIDATCON_DATASIZE  ((cyg_uint32) 22)
#define bwSDI_SDIDATCON_DATASIZE  ((cyg_uint32)  2)
#define bsSDI_SDIDATCON_BURST4    ((cyg_uint32) 24)
#define bwSDI_SDIDATCON_BURST4    ((cyg_uint32)  1)


/*
 * Description:
 * Bit offsets and masks for SDI Data Remain Register (SDIDATCNT)
 */
#define bsSDI_SDIDATCNT_BLKCNT     ((cyg_uint32)  0)
#define bwSDI_SDIDATCNT_BLKCNT     ((cyg_uint32) 12)
#define bsSDI_SDIDATCNT_BLKNUMCNT  ((cyg_uint32) 12)
#define bwSDI_SDIDATCNT_BLKNUMCNT  ((cyg_uint32) 12)


/*
 * Description:
 * Bit offsets and masks for SDI Data Status Register (SDIDATSTA)
 */
#define SDI_SDIDATSTA_RXDATON     ((cyg_uint32) (1<< 0))
#define SDI_SDIDATSTA_TXDATON     ((cyg_uint32) (1<< 1))
#define SDI_SDIDATSTA_BUSYFIN     ((cyg_uint32) (1<< 3))
#define SDI_SDIDATSTA_DATFIN      ((cyg_uint32) (1<< 4))
#define SDI_SDIDATSTA_DATTOUT     ((cyg_uint32) (1<< 5))
#define SDI_SDIDATSTA_DATCRC      ((cyg_uint32) (1<< 6))
#define SDI_SDIDATSTA_CRCSTA      ((cyg_uint32) (1<< 7))
#define SDI_SDIDATSTA_IOINTDET    ((cyg_uint32) (1<< 9))
#define SDI_SDIDATSTA_RWAITREQ    ((cyg_uint32) (1<<10))
#define SDI_SDIDATSTA_NOBUSY      ((cyg_uint32) (1<<11))

/*
 * Description:
 * Bit offsets and masks for SDI FIFO Status Register (SDIFSTA)
 */
#define bsSDI_SDIFSTA_FFCNT         ((cyg_uint32)  0)
#define bwSDI_SDIFSTA_FFCNT         ((cyg_uint32)  7)
#define bsSDI_SDIFSTA_FFFAIL        ((cyg_uint32) 14)
#define bwSDI_SDIFSTA_FFFAIL        ((cyg_uint32)  2)

#define SDI_SDIFSTA_RFHALF          ((cyg_uint32) (1<< 7))
#define SDI_SDIFSTA_RFFULL          ((cyg_uint32) (1<< 8))
#define SDI_SDIFSTA_RFLAST          ((cyg_uint32) (1<< 9))
#define SDI_SDIFSTA_TFEMPTY         ((cyg_uint32) (1<<10))
#define SDI_SDIFSTA_TFHALF          ((cyg_uint32) (1<<11))
#define SDI_SDIFSTA_RFDET           ((cyg_uint32) (1<<12))
#define SDI_SDIFSTA_TFDET           ((cyg_uint32) (1<<13))
#define SDI_SDIFSTA_FRST            ((cyg_uint32) (1<<15))


/*
 * Description:
 * Bit offsets and masks for SDI Interrupt Mask Register (SDINTMSK)
 */
#define SDI_SDINTMSK_RFHALFINT     ((cyg_uint32) (1<<0))
#define SDI_SDINTMSK_RFFULLINT     ((cyg_uint32) (1<<1))
#define SDI_SDINTMSK_RFLASTINT     ((cyg_uint32) (1<<2))
#define SDI_SDINTMSK_TFEMPTINT     ((cyg_uint32) (1<<3))
#define SDI_SDINTMSK_TFHALFINT     ((cyg_uint32) (1<<4))
#define SDI_SDINTMSK_BUSYFININT    ((cyg_uint32) (1<<6))
#define SDI_SDINTMSK_DATFININT     ((cyg_uint32) (1<<7))
#define SDI_SDINTMSK_DATTOUTINT    ((cyg_uint32) (1<<8))
#define SDI_SDINTMSK_DATCRCINT     ((cyg_uint32) (1<<9))
#define SDI_SDINTMSK_CRCSTAINT     ((cyg_uint32) (1<<10))
#define SDI_SDINTMSK_FFFAILINT     ((cyg_uint32) (1<<11))
#define SDI_SDINTMSK_INTDETINT     ((cyg_uint32) (1<<12))
#define SDI_SDINTMSK_RWREQINT      ((cyg_uint32) (1<<13))
#define SDI_SDINTMSK_RSPENDINT     ((cyg_uint32) (1<<14))
#define SDI_SDINTMSK_CMDTOUTINT    ((cyg_uint32) (1<<15))
#define SDI_SDINTMSK_CMDSENTINT    ((cyg_uint32) (1<<16))
#define SDI_SDINTMSK_RSPCRCINT     ((cyg_uint32) (1<<17))
#define SDI_SDINTMSK_NOBUSYINT     ((cyg_uint32) (1<<18))


#define SDI_DATA_INTMASK \
        (SDI_SDINTMSK_DATFININT  | \
         SDI_SDINTMSK_DATTOUTINT | \
         SDI_SDINTMSK_DATCRCINT  | \
         SDI_SDINTMSK_CRCSTAINT  | \
         SDI_SDINTMSK_RSPCRCINT)

#define SDI_FIFO_INTMASK \
        (SDI_SDINTMSK_RFHALFINT | \
         SDI_SDINTMSK_RFFULLINT | \
         SDI_SDINTMSK_RFLASTINT | \
         SDI_SDINTMSK_TFEMPTINT | \
         SDI_SDINTMSK_FFFAILINT)

#define SDI_CMD_INTMASK \
        (SDI_SDINTMSK_CRCSTAINT  | \
         SDI_SDINTMSK_RWREQINT   | \
         SDI_SDINTMSK_RSPENDINT  | \
         SDI_SDINTMSK_CMDTOUTINT | \
         SDI_SDINTMSK_CMDSENTINT | \
         SDI_SDINTMSK_RSPCRCINT)


#define SDI_ALL_INTMASK (SDI_CMD_INTMASK | SDI_DATA_INTMASK | SDI_FIFO_INTMASK)
#define SDI_DEFAULT_INTMASK SDI_ALL_INTMASK
#define SDI_DATA_CMD_INTMASK  (SDI_CMD_INTMASK | SDI_DATA_INTMASK)

/*we must ensure a clock speed below 400kHz from a base clock up to 20MHz*/
#define SDI_MCLK_SPEED          PCLK       // 61.750.000Hz
#define SDI_FREQ_SLOW           300000     // 300kHz
#define SDI_FREQ_SDCARD         (PCLK/3)   // 20.583.333Hz
#define SDI_FREQ_MMC            (PCLK/4)   // 15.437.500Hz
#define SDI_FREQ_FULL           SDI_FREQ_SDCARD
#define SDI_FREQ_HIGH           (PCLK/2)   // 30.875.000Hz

#ifndef MIN
#define MIN(_x,_y) (((_x)<(_y))?(_x):(_y))
#endif

typedef enum SDI_xResponseType {
    SDI_RESPONSE_NONE    = 0x00,
    SDI_RESPONSE_SHORT   = 0x01,
    SDI_RESPONSE_LONG    = 0x03
} SDI_eResponseType;

typedef enum SDI_xCmdState {
    SDI_STATE_CMD_NOINIT,     // unintialised
    SDI_STATE_CMD,            // processing a command
} SDI_eCmdState;

typedef enum SDI_xDataState {
    SDI_STATE_DATA_NOINIT,
    SDI_STATE_DATA_SENDING,
    SDI_STATE_DATA_RECEIVING
} SDI_eDataState;

typedef enum SDI_xError {
    SDI_ERROR_NONE = 0,
    SDI_ERROR_CRCFAIL,
    SDI_ERROR_TIMEOUT,
    SDI_ERROR_DATA_TIMEOUT,
    SDI_ERROR_DATA_HW_ERROR,
    SDI_ERROR_DATA_FIFO_UNDERRUN //or overrun
} SDI_eError;

typedef enum SDI_xDataMode {
    SDI_DATAMODE_BLOCK = 0,
    SDI_DATAMODE_STREAM = 1
} SDI_eDataMode;

typedef volatile struct SDI_xStateStruct {
    volatile cyg_uint32 * Base;
    SDI_eCmdState eCmdState;
    SDI_eResponseType cmd_response_type;
    cyg_uint32 * cmd_response_index;
    cyg_uint32 * cmd_response;
    SDI_eDataState eDataState;
    SDI_eError eLastCmdError;
    SDI_eError eLastDataError;
    cyg_uint32 BlockLength;
    cyg_uint8 * data; // data to send or received
    cyg_uint32 length; // number of bytes to send or received
    cyg_uint32 it_count; // number of IT
    cyg_bool wide_bus;
} SDI_sStateStruct;

#define SDI_MAX_TRANSFER_SIZE (32*1024) // must be a multiple of 512

extern SDI_sStateStruct SDI_State;


void SDI_Init(void);
void SDI_PowerOn(void);
void SDI_ClockEnable(cyg_bool b);
void SDI_SetClockFreq(cyg_uint32 freq);
cyg_uint32 SDI_GetClockFreq(void);
void SDI_SetWideBus(cyg_bool wide);
SDI_eError SDI_SendCommand(cyg_uint32 command_index, cyg_uint32 argument, SDI_eResponseType response_type,
                cyg_uint32 * response_index, cyg_uint32 * response);
SDI_eError SDI_SendData(const cyg_uint8 * data, cyg_uint32 length, SDI_eDataMode data_mode);
SDI_eError SDI_OpenReceiveWindow(cyg_uint8 * data, cyg_uint32 length, SDI_eDataMode data_mode);
SDI_eError SDI_CloseReceiveWindow(void);

#endif
