//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file  sdi_dma.h
* @brief DMA functions
*
* Copyright (C) 2007 Parrot S.A.
*
* @author     florent.bayendrian@parrot.fr
* @date       2007-02-27
*******************************************************************************
*/

#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_io.h> // HAL_WRITE_UINT32
#include <cyg/utils/utils.h>
#include <assert.h>
#include "sdcard.h"
#include "sdi_dma.h"
#include "sdi.h"

#define SDI_DMA_INT_PRIORITY 4
#define S3C2412_DMA_REQSRC_SDMMC 10

#define S3C2412_POLL_TIMEOUT 1000000

/**
 * Configure a DMA transfer from/to SDI interface
 *
 * @param vaddr virtual start address of the transfer
 * @param size size of the transfer
 * @param receiving direction of the transfer
 */
void sdi_dma_devconfig(cyg_uint8 * vaddr, cyg_uint32 size, cyg_bool receiving)
{
    // physical address
    cyg_uint32 * paddr = (cyg_uint32*) arm_mmu_v2p((CYG_ADDRESS) vaddr);
    cyg_uint32 reg;

    // vaddr must be mapped
    assert(paddr);

    reg = DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG);
    // disable DMA if on
    if (reg & S3C2412_DMA_DMASKTRIG_ON) {
        DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG) = S3C2412_DMA_DMASKTRIG_STOP;
        while (DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG) & S3C2412_DMA_DMASKTRIG_ON); 
        DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG) = 0;
    }

    CYG_ASSERT(DMA_REGISTER(S3C2412_DMA_REG_DSTAT) == 0, "DMA unavailable !!");

    if (receiving) {
        cyg_uint32 reg_dcon = 0;
        cyg_uint32 reg_dmareqsel = 0;

        // source = FIFO ; physical address equals virtual address
        DMA_REGISTER(S3C2412_DMA_REG_DISRC) = (cyg_uint32) &SDI_REGISTER(SDI_REG_SDIDAT);
        // source is in peripheral bus ; address is fixed (FIFO)
        DMA_REGISTER(S3C2412_DMA_REG_DISRCC) = S3C2412_DMA_DISRCC_LOC | S3C2412_DMA_DISRCC_INC;
        // destination = memory ; address is increased after each transfer
        DMA_REGISTER(S3C2412_DMA_REG_DIDST) = (cyg_uint32) paddr;
        DMA_REGISTER(S3C2412_DMA_REG_DIDSTC) = 0;
        // set S3C2412_DMA_REG_DMAREQSEL
        BIT_SET(reg_dmareqsel, S3C2412_DMA_DMAREQSEL_HWSRCSEL, S3C2412_DMA_REQSRC_SDMMC);
        reg_dmareqsel |= S3C2412_DMA_DMAREQSEL_SWHW_SEL;
        DMA_REGISTER(S3C2412_DMA_REG_DMAREQSEL) = reg_dmareqsel;
        // set S3C2412_DMA_REG_DCON
        BIT_SET(reg_dcon, S3C2412_DMA_DCON_DSZ, 0x2); // word transfer
        BIT_SET(reg_dcon, S3C2412_DMA_DCON_TC, size>>2); // word transfer
        reg_dcon |= S3C2412_DMA_DCON_RELOAD |
                    S3C2412_DMA_DCON_DMD_HS;
        DMA_REGISTER(S3C2412_DMA_REG_DCON) = reg_dcon;
        DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG) = S3C2412_DMA_DMASKTRIG_ON;
    } else {
        cyg_uint32 reg_dcon = 0;
        cyg_uint32 reg_dmareqsel = 0;

        arm_mmu_clean_dcache_range((CYG_ADDRESS)vaddr, size);

        // source = memory ; address is increased after each transfer
        DMA_REGISTER(S3C2412_DMA_REG_DISRC) = (cyg_uint32) paddr;
        DMA_REGISTER(S3C2412_DMA_REG_DISRCC) = 0;
        // destination = FIFO ; physical address equals virtual address
        DMA_REGISTER(S3C2412_DMA_REG_DIDST) = (cyg_uint32) &SDI_REGISTER(SDI_REG_SDIDAT);
        DMA_REGISTER(S3C2412_DMA_REG_DIDSTC) = S3C2412_DMA_DISRCC_LOC | S3C2412_DMA_DISRCC_INC;
        // set S3C2412_DMA_REG_DMAREQSEL
        BIT_SET(reg_dmareqsel, S3C2412_DMA_DMAREQSEL_HWSRCSEL, S3C2412_DMA_REQSRC_SDMMC);
        reg_dmareqsel |= S3C2412_DMA_DMAREQSEL_SWHW_SEL;
        DMA_REGISTER(S3C2412_DMA_REG_DMAREQSEL) = reg_dmareqsel;
        // set S3C2412_DMA_REG_DCON
        BIT_SET(reg_dcon, S3C2412_DMA_DCON_DSZ, 0x2); // word transfer
        BIT_SET(reg_dcon, S3C2412_DMA_DCON_TC, size>>2); // word transfer
        reg_dcon |= S3C2412_DMA_DCON_RELOAD |
                    S3C2412_DMA_DCON_DMD_HS;
        DMA_REGISTER(S3C2412_DMA_REG_DCON) = reg_dcon;
        DMA_REGISTER(S3C2412_DMA_REG_DMASKTRIG) = S3C2412_DMA_DMASKTRIG_ON;
    }
}

/*
 * Wait for the last DMA xfer
 */
Cyg_ErrNo sdi_dma_finish(void)
{
    cyg_uint32 timeout = S3C2412_POLL_TIMEOUT;
    int ret = ENOERR;

    while (BIT_GET(DMA_REGISTER(S3C2412_DMA_REG_DSTAT), S3C2412_DMA_DSTAT_STAT) != 0
            && timeout--);

    if (timeout == 0) {
        cyg_uint32 reg = DMA_REGISTER(S3C2412_DMA_REG_DSTAT);
        reg |= S3C2412_DMA_DMASKTRIG_STOP;
        DMA_REGISTER(S3C2412_DMA_REG_DSTAT) = reg;
        DEBUG2("DMA timeout!!\n");
        ret = -ETIMEDOUT;
    }

    return ret;
}

