//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ***************************************************************************
 * @file sta3xx.h
 *
 * @brief STA3XX digital amplifier driver.
 *
 * Copyright (C) 2007 Parrot S.A.
 *
 * @author	Frederic CHANAL
 * @date	05/2007
 ***************************************************************************
 */

#include <cyg/infra/cyg_ass.h>
#include <cyg/devs/sta3xx.h>


#ifdef CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW

/* STA335BW I2C devices defintions */
CYG_I2C_DEVICE(STA3XX_i2c_device1,	\
			   NULL,					\
			   0x38>>1,					\
			   0x00,					\
			   CYG_I2C_DEFAULT_DELAY);
			   
CYG_I2C_DEVICE(STA3XX_i2c_device2,	\
			   NULL,					\
			   0x3A>>1,					\
			   0x00,					\
			   CYG_I2C_DEFAULT_DELAY);
	
cyg_i2c_device *STA3XX_i2c_device[] =
{
	[STA3XX_STA335BW_0] = &STA3XX_i2c_device1,
	[STA3XX_STA335BW_1] = &STA3XX_i2c_device2,
};
	
#elif defined(CYGPKG_DEVICE_STA3XX_SETTINGS_STA326)
CYG_I2C_DEVICE(STA3XX_i2c_device1,	\
			   NULL,					\
			   0x34>>1,					\
			   0x00,					\
			   CYG_I2C_DEFAULT_DELAY);

cyg_i2c_device *STA3XX_i2c_device[] =
{
	[STA3XX_STA326] = &STA3XX_i2c_device1,
};
	
#endif

cyg_uint8 STA3XX_mute[STA3XX_DEVICE_COUNT];		///< Mute state
cyg_mutex_t STA3XX_mute_mutex;					///< Mutex to access STA3XX_mute


/** Initialize the amplifier.
 * This function should be used on startup, to configure the amplifier. It
 * does not perform a reset, so the caller should take care of that before
 * calling the function, to ensure the parameters are properly set.
 *
 * Once the function has been called, the other APIs can be used to perform
 * other commands (setup, volume change...).
 *
 * @param config A structure containing the configuration of the amplifier.
 * This structure is used only during the call, it does not need to be valid
 * afterward.
 */
void STA3XX_Init(STA3XX_DEVICE device, cyg_i2c_bus *bus, const STA3XX_CONFIG *config)
{
	cyg_uint8 cfgbuf[]= {
		config->confA,
		config->confB,
		config->confC,
		config->confD,
		config->confE,
		config->confF,
		STA3XX_MASTERMUTE,	//master mute is set by default
		config->volumeMaster,
		config->volumeChan1,
		config->volumeChan2,
		config->volumeChan3
	};

	CYG_ASSERT(bus, "NULL pointer arguments: bus");
	CYG_ASSERT(config, "NULL pointer arguments: config");

	cyg_mutex_init(&STA3XX_mute_mutex);
	STA3XX_mute[device] = STA3XX_MASTERMUTE;
	
	(*STA3XX_i2c_device[device]).i2c_bus = bus;
	STA3XX_Write(device, STA3XX_REG_ConfA, cfgbuf, sizeof(cfgbuf));
}

void STA3XX_InitSimple(STA3XX_DEVICE device, cyg_i2c_bus *bus)
{
	CYG_ASSERT(bus, "NULL pointer arguments: bus");
	
	cyg_mutex_init(&STA3XX_mute_mutex);
	STA3XX_mute[device] = STA3XX_MASTERMUTE;
	
	(*STA3XX_i2c_device[device]).i2c_bus = bus;
}

/** Mute any of the channels.
 * @param mute A combination of the following flags:
 * 			- STA3XX_MUTE_MASTER mute all channels
 *			- STA3XX_MUTE_CHAN1 mute channel 1
 * 			- STA3XX_MUTE_CHAN2 mute channel 2
 * 			- STA3XX_MUTE_CHAN3 mute channel 3
 *
 * To unmute, call the STA3XX_UnMute() function.
 * A channel is actually muted if STA3XX_MUTE_CHAN<x> or STA3XX_MUTE_MASTER
 * is set. To unmute it, both flags must be passed to the unmute function.
 */
void STA3XX_Mute(STA3XX_DEVICE device, cyg_uint8 mute)
{
	cyg_mutex_lock(&STA3XX_mute_mutex);
	if ((STA3XX_mute[device] | mute) != STA3XX_mute[device])
	{
		STA3XX_mute[device] |= mute;
		STA3XX_Write(device, STA3XX_REG_Mute, &STA3XX_mute[device], 1);
	}
	cyg_mutex_unlock(&STA3XX_mute_mutex);
}

/** Unmute any of the channels.
 * @param mute A combination of the following flags:
 * 			- STA3XX_MUTE_MASTER unmute all channels
 *			- STA3XX_MUTE_CHAN1 unmute channel 1
 * 			- STA3XX_MUTE_CHAN2 unmute channel 2
 * 			- STA3XX_MUTE_CHAN3 unmute channel 3
 *
 * A channel is actually muted if STA3XX_MUTE_CHAN<x> or STA3XX_MUTE_MASTER
 * is set. To unmute it, both flags must be passed to the function.
 */
void STA3XX_UnMute(STA3XX_DEVICE device, cyg_uint8 mute)
{
	cyg_mutex_lock(&STA3XX_mute_mutex);
	if (STA3XX_mute[device] & mute)
	{
		STA3XX_mute[device] &= ~mute;
		STA3XX_Write(device, STA3XX_REG_Mute, &STA3XX_mute[device], 1);
	}
	cyg_mutex_unlock(&STA3XX_mute_mutex);
}

void STA3XX_Write(STA3XX_DEVICE device, cyg_uint16 addr, const cyg_uint8 * buf, cyg_uint8 len)
{
	cyg_uint8 tx_data[1];

	CYG_ASSERT(STA3XX_i2c_device[device]->i2c_bus, "Call STA3XX_Init() first");
	CYG_ASSERT(buf, "NULL pointer arguments: buf");

	// address:
	tx_data[0] = addr;

	// lock
	cyg_i2c_transaction_begin(STA3XX_i2c_device[device]);

	// send the address: start, no stop
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, tx_data, 1, false);
	// send the data: start, stop
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], false, buf, len, true);

	// unlock
	cyg_i2c_transaction_end(STA3XX_i2c_device[device]);
}

void STA3XX_Read(STA3XX_DEVICE device, cyg_uint16 addr, cyg_uint8 * buf , cyg_uint8 len )
{
	cyg_uint8 tx_data[1];

	CYG_ASSERT(STA3XX_i2c_device[device]->i2c_bus, "Call STA3XX_Init() first");
	CYG_ASSERT(buf, "NULL pointer arguments: buf");

	// address:
	tx_data[0] = addr;

	// lock
	cyg_i2c_transaction_begin(STA3XX_i2c_device[device]);

	// send the address: start, no stop
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, tx_data, 1, false);
	// receive the data: start, stop
	cyg_i2c_transaction_rx(STA3XX_i2c_device[device], true, buf, len, true, true);

	// unlock
	cyg_i2c_transaction_end(STA3XX_i2c_device[device]);
}

void STA3XX_Coeff_Write(STA3XX_DEVICE device, STA3XX_Coeff coeff, int value)
{
	cyg_uint8 command[20];
	int pos = 0;

	CYG_ASSERT(STA3XX_i2c_device[device]->i2c_bus, "Call STA3XX_Init() first");

	//lock
	cyg_i2c_transaction_begin(STA3XX_i2c_device[device]);

	// First, send the coefficient
	command[0] = STA3XX_REG_Cfaddr;
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, command, 1, false);

	command[pos++] = coeff;
	CYG_ASSERT(!(value >> 24) || !(~(value >> 24)), "coeffs are 24bit.");
	command[pos++] = (value >> 16) & 0xff;
	command[pos++] = (value >> 8) & 0xff;
	command[pos++] = (value) & 0xff;
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], false, command, pos, true);

	// Then, send write command
	command[0] = STA3XX_REG_Cfud;
	command[1] = STA3XX_Coeff_WriteOne;
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, command, 2, true);

#ifdef CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW

	/* HACK: STA335 need command WriteOne to be written 3 times in a row, in order to 
	 * ensure that coefficients are properly written inside internal RAM
	 */
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, command, 2, true);
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, command, 2, true);

#endif // CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW

	// unlock
	cyg_i2c_transaction_end(STA3XX_i2c_device[device]);
}

void STA3XX_Coeff_Read(STA3XX_DEVICE device, STA3XX_Coeff coeffadd, int *value)
{
	cyg_uint8 command[20];

	CYG_ASSERT(STA3XX_i2c_device[device]->i2c_bus, "Call STA3XX_Init() first");

	//lock
	cyg_i2c_transaction_begin(STA3XX_i2c_device[device]);

	// Write 6-bits of address to I2C register STA3XX_REG_Cfaddr
	command[0] = STA3XX_REG_Cfaddr;
	command[1] = coeffadd;
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, command, 2, true);

	// Then, write R1 to STA3XX_REG_Cfud
	command[0] = STA3XX_REG_Cfud;
	command[1] = STA3XX_Coeff_ReadOne;
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, command, 2, true);

	// Then read coeff
	command[0] = STA3XX_REG_B1cf1;
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, command, 1, false);
	cyg_i2c_transaction_rx(STA3XX_i2c_device[device], true, command, 3, true, true);

	*value = (command[0] << 16) | (command[1] << 8) | command[2];
	
	// unlock
	cyg_i2c_transaction_end(STA3XX_i2c_device[device]);
	
}

void STA3XX_Coeff_WriteBiquad(STA3XX_DEVICE device, STA3XX_Coeff_BIQUAD biquad, int b1, int b2, int a1, int a2, int b0)
{
	cyg_uint8 command[20];
	int coeffs[5] = { b1, b2, a1, a2, b0 };
	int pos = 0;
	cyg_uint8 tx_data[2];
	int i;

	command[pos++] = biquad;
	for(i=0; i<5; i++)
	{
		CYG_ASSERT(!(coeffs[i] >> 24) || !(~(coeffs[i] >> 24)), "coeffs are 24bit.");
		command[pos++] = (coeffs[i] >> 16) & 0xff;
		command[pos++] = (coeffs[i] >> 8) & 0xff;
		command[pos++] = (coeffs[i]) & 0xff;
	}

#ifdef CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW
	/* HACK: STA335 need command WriteAll to be written 3 times in a row, in order to 
	 * ensure that coefficients are properly written inside internal RAM
	 */

	CYG_ASSERT(STA3XX_i2c_device[device]->i2c_bus, "Call STA3XX_Init() first");

	// address:
	tx_data[0] = STA3XX_REG_Cfaddr;

	// lock
	cyg_i2c_transaction_begin(STA3XX_i2c_device[device]);

	// send the address: start, no stop
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, tx_data, 1, false);
	// send the data: start, stop
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], false, command, pos, true);
	
	// send 3 time the WriteAll command
	tx_data[0] = STA3XX_REG_Cfud;
	tx_data[1] = STA3XX_Coeff_WriteAll;
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, tx_data, 2, true);
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, tx_data, 2, true);
	cyg_i2c_transaction_tx(STA3XX_i2c_device[device], true, tx_data, 2, true);

	// unlock
	cyg_i2c_transaction_end(STA3XX_i2c_device[device]);

#else //CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW

	command[pos++] = STA3XX_Coeff_WriteAll;
	STA3XX_Write(device, STA3XX_REG_Cfaddr, command, pos);

#endif // CYGPKG_DEVICE_STA3XX_SETTINGS_STA335BW

}

