//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 ***************************************************************************
 * @file tda7540.h
 *
 * @brief TDA7540 AM/FM Car Radio Tuner IC.
 *
 * Copyright (C) 2007 Parrot S.A.
 *
 * @author	Yoan Besson
 * @date	03/2007
 ***************************************************************************
 */

#include <cyg/infra/cyg_ass.h>
#include <cyg/infra/diag.h>         // diagnostic output
#include <cyg/devs/tda7540.h>
#include <stdio.h>

// support for the debug
#if 0
# define TDA7540_DEBUG(format, ...) diag_printf(format, ## __VA_ARGS__)
#else
# define TDA7540_DEBUG(_format_, ...)
#endif

// debug display eeprom content
#if 0
# define TDA7540_DEBUG_DISPLAY_EEPROM(istart, istop, mode) TDA7540_DebugDisplayTunerBuffer(istart, istop, mode)
# define TDA7540_DEBUG_DISPLAY_REG(istart, istop, mode) TDA7540_DebugDisplayTunerBuffer(istart, istop, mode)
#else
# define TDA7540_DEBUG_DISPLAY_EEPROM(istart, istop, mode) TDA7540_DEBUG("EEPROM read ok\n")
# define TDA7540_DEBUG_DISPLAY_REG(istart, istop, mode)
#endif

/** AM/FM Car Radio Tuner I2C device.
 */
CYG_I2C_DEVICE(TDA7540_i2c_device,		\
			   NULL,					\
			   0xC4>>1,					\
			   0x00,					\
			   CYG_I2C_DEFAULT_DELAY);

/** EEPROM I2C device containing init tuner data.
 */
CYG_I2C_DEVICE(TunerEEPROM_i2c_device,	\
			   NULL,					\
			   0x00,					\
			   0x00,					\
			   CYG_I2C_DEFAULT_DELAY);

#ifndef TDA7540_USE_EEPROM

#define TDA7540_COMPLETE_WITH_EEPROM

#define TDA7540_CUSTOM_EEPROM_P0 0
#define TDA7540_CUSTOM_EEPROM_ST 9

#define TDA7540_CUSTOM_EEPROM_VALUES		TDA7540_CUSTOM_EEPROM_ST

#if (TDA7540_CUSTOM_EEPROM_VALUES == TDA7540_CUSTOM_EEPROM_P0)
	const cyg_uint8 TDA7540_EEPROM[]=
	{
		0x0c,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0xd6,
		0x60,
		0x08,
		0x00,
		0xde,
		0x13,
		0x8f,
		0x00,
		0x60,
		0xfe,
		0x00,
		0x44,
		0xfa,
		0xc7,
		0x25,
		0x60,
		0xe8,
		0xc0,
		0xe0,
		0x97,
		0x68,
		0xc8,
		0x08,
		0x60,
		0x47,
		0x5c,
		0xe7,
		0x7f,
		0x7f,
		0xdc,
		0x1d,
		0x7d,
		0x00,
		0x00,
		0x00,
		0x00,
		0x00,
		0x00,
		0xfe,
		0xbe,
		0xe6,
		0xde,
		0xe2,
		0x47,
		0x4d,
		0x4d,
		0xe4,
		0xe0,
		0xdf,
		0xe1
	};
#elif (TDA7540_CUSTOM_EEPROM_VALUES == TDA7540_CUSTOM_EEPROM_ST)
	const cyg_uint8 TDA7540_EEPROM[]=
	{
		0x0c,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0,
		0xD0,
		0xAA,
		0x07,
		0x00,
		0x0B,
		0x93,
		0xAF,
		0x80,
		0x61,
		0x32,
		0x20,
		0x5A,
		0xF8,
		0xFF,
		0x25,
		0x60,
		0x00,
		0x7F,
		0xB0,
		0xC7,
		0xA0,
		0xC3,
		0x00,
		0x00,
		0xA7,
		0x0E,
		0x20,
		0x6B,
		0x60,
		0x48,
		0x81,
		0xFD,
		0x04,
		0x00,
		0x00,
		0x00,
		0x00,
		0x00,
		0xFE,
		0xFE,
	};
	#endif
#endif

static cyg_uint8 TDA7540Buffer[TDA7540_BUFFER_SIZE];		///< Address MAP.
static cyg_uint8 TDA7540TV2Buffer[TDA7540_TV2_BUFFER_SIZE];	///< Address MAP.

cyg_mutex_t TDA7540Mutex;	///< Mutex to access TDA7540Mute

int TDA7540_WriteOnPage(cyg_uint8 addr, const cyg_uint8* buf, cyg_uint8 len);

void TDA7540_GetBuffer(cyg_uint8** ioTunerBuffer);
void TDA7540_GetBufferReadOnly(const cyg_uint8** ioTunerBuffer);
void TDA7540_GetTV2Buffer(cyg_uint8** ioTunerTV2Buffer);
void TDA7540_GetTV2BufferReadOnly(cyg_uint8** ioTunerTV2Buffer);
void TDA7540_ReleaseBuffer(void);

void TDA7540_DebugDisplayTunerBuffer(cyg_uint16 istart, cyg_uint16 istop, int mode);

void test(void)
{
	cyg_uint8 request[2];	// Buffer de la requete
	cyg_uint8 len_request = 0;		// Taille du buffer de la requete

	len_request = 2;
	request[0] = 3;
	request[1] = 0xFF;

	/* lock the I2C bus */
	cyg_i2c_transaction_begin(&TDA7540_i2c_device);
	cyg_i2c_transaction_tx(&TDA7540_i2c_device, true, request, len_request, true);

	len_request = 2;
}

/** Initialize the tuner.
 * This function should be used on startup, to configure the tuner. As TDA7540
 * is used in tuner modules (like KST-120 or TBK TC-511), registers will be set
 * by I2C at start up using data stored in an EEPROM. The EEPROM I2C address as to
 * sent here.
 *
 * Once the function has been called, the other APIs can be used to perform
 * other commands.
 *
 * @param[in] busTDA		Bus to attach TDA7540 device to.
 * @param[in] busEEPROM		Bus to attach associated EEPROM to.
 * @param[in] addrEEPROM	EEPROM I2C address.
 */
void TDA7540_Init(cyg_i2c_bus* busTDA, cyg_i2c_bus* busEEPROM, cyg_uint16 addrEEPROM)
{
	CYG_ASSERT(busTDA, "NULL pointer arguments: busTDA");
	CYG_ASSERT(busEEPROM, "NULL pointer arguments: busEEPROM");

	cyg_mutex_init(&TDA7540Mutex);
	
	TDA7540_i2c_device.i2c_bus = busTDA;
	TunerEEPROM_i2c_device.i2c_bus = busEEPROM;
	TunerEEPROM_i2c_device.i2c_address = addrEEPROM>>1;
}

void TDA7540_Uninit(void)
{
	cyg_mutex_destroy(&TDA7540Mutex);
}


//////////////////////////////////////////////////////////////////////////
//	I2C
//////////////////////////////////////////////////////////////////////////

cyg_bool TDA7540_CheckI2cComm(void)
{
	cyg_bool ack;
	cyg_uint8 tx_data = 0;
	
	// lock
	cyg_i2c_transaction_begin(&TDA7540_i2c_device);

	// send the address: start, stop
	ack = cyg_i2c_transaction_tx(&TDA7540_i2c_device, true, &tx_data, 1, true);

	// unlock
	cyg_i2c_transaction_end(&TDA7540_i2c_device);

	return ack;
}

int TDA7540_Write(cyg_uint8 addr, const cyg_uint8* buf, cyg_uint8 len)
{
	cyg_uint8 lendata = len;
	int result = 0;

	CYG_ASSERT(TDA7540_i2c_device.i2c_bus, "Call TDA7540_Init() first");
	CYG_ASSERT(buf, "NULL pointer arguments: buf");

	// Calculate length if data will be sent in two times (if addresses exceeds first page).
	if (addr < 32)
		lendata = (len<32-addr)?len:(32-addr);

	// Send First data part.
	result = TDA7540_WriteOnPage(addr, buf, lendata);

	// Send Second data part.
	if (result == lendata && lendata != len)
		result = TDA7540_WriteOnPage(addr+lendata, buf+lendata, len-lendata);

	return result;
}

int TDA7540_WriteOnPage(cyg_uint8 addr, const cyg_uint8* buf, cyg_uint8 len)
{
	cyg_uint8 tx_data[1];
	int result = -1;
	int attempt = 0;

	CYG_ASSERT(TDA7540_i2c_device.i2c_bus, "Call TDA7540_Init() first.");
	CYG_ASSERT(buf, "NULL pointer arguments: buf.");
	CYG_ASSERT(((addr&TDA7540_AUTO_INC)!=TDA7540_AUTO_INC), "pagermode bit set. If auto-increasing is required, it will be added automatically.");
//	CYG_ASSERT((addr+len)<=32, "All data can't be written in one time, use TDA7540_Write().");

	// address:
	tx_data[0] = addr;

	// Add "auto-increase" value if more than one byte will be sent.
	if (len > 1)
		tx_data[0] += TDA7540_AUTO_INC;

	// lock
	cyg_i2c_transaction_begin(&TDA7540_i2c_device);

	while (result < 0 && attempt < 3)
	{
		// send the address: start, no stop
		result = cyg_i2c_transaction_tx(&TDA7540_i2c_device, true, tx_data, 1, false);
		if (result < 1)
			result = -1;
		else
		{
			// send the data: no start, stop
			result = cyg_i2c_transaction_tx(&TDA7540_i2c_device, false, buf, len, true);
			if (result < len)
				result = -1;
		}

		// if an error occured try again 3 times.
		if (result == -1)
		{
			TDA7540_DEBUG("attempt %d failed, try again...\n", attempt);
			attempt++;
		}
	}

	// unlock
	cyg_i2c_transaction_end(&TDA7540_i2c_device);

	return result;
}

void TDA7540_Read(cyg_uint8* buf)
{
	cyg_uint8 tx_data[1];

	CYG_ASSERT(TDA7540_i2c_device.i2c_bus, "Call TDA7540_Init() first");
	CYG_ASSERT(buf, "NULL pointer arguments: buf");

	// address:
	tx_data[0] = 0;

	// lock
	cyg_i2c_transaction_begin(&TDA7540_i2c_device);

	// send the address: start, no stop
	cyg_i2c_transaction_tx(&TDA7540_i2c_device, true, tx_data, 1, true);
	// receive the data: start, stop
	cyg_i2c_transaction_rx(&TDA7540_i2c_device, true, buf, 1, true, true);

	// unlock
	cyg_i2c_transaction_end(&TDA7540_i2c_device);
}

cyg_bool TDA7540_ReadEEPROM(cyg_uint8 addr, cyg_uint8* buf, cyg_uint8 len)
{
	cyg_bool ack = true;
	cyg_uint8 tx_data[1];

#ifndef TDA7540_USE_EEPROM
	cyg_uint8 len2 = len;
	cyg_uint8 addr2 = addr;

	if ((addr+len) >= sizeof(TDA7540_EEPROM))
	{
		if (addr < sizeof(TDA7540_EEPROM))
		{
			len = ((addr+len) - sizeof(TDA7540_EEPROM));
			addr = sizeof(TDA7540_EEPROM);
			len2 -= len;
		}
		else
		{
			len2 = 0;
		}
	}
	else
	{
		len = 0;
	}

	if (len > 0)
	{
#endif

	CYG_ASSERT(buf, "NULL pointer arguments: buf");

#if (defined(TDA7540_USE_EEPROM) || defined(TDA7540_COMPLETE_WITH_EEPROM))
	// address:
	tx_data[0] = addr;

	// lock
	cyg_i2c_transaction_begin(&TunerEEPROM_i2c_device);

	// send the address: start, stop
	ack = cyg_i2c_transaction_tx(&TunerEEPROM_i2c_device, true, tx_data, 1, true);

	if (ack)
	{
		// receive the data: nack, stop
		cyg_i2c_transaction_rx(&TunerEEPROM_i2c_device, true, buf, len, true, true);
	}

	// unlock
	cyg_i2c_transaction_end(&TunerEEPROM_i2c_device);
#endif

#ifndef TDA7540_USE_EEPROM
	}
	if (len2)
	{
		memcpy(buf, &(TDA7540_EEPROM[addr2]), len2);
	}
#endif

	return ack;
}

void TDA7540_WriteEEPROM(cyg_uint8 addr, const cyg_uint8* buf, cyg_uint8 len)
{
	cyg_uint8 tx_data[1];

#ifndef TDA7540_USE_EEPROM
	if ((addr+len) >= sizeof(TDA7540_EEPROM))
	{
		len = ((addr+len) - sizeof(TDA7540_EEPROM) + 1);
		addr = sizeof(TDA7540_EEPROM);
	}
	else
	{
		len = 0;
	}

	if (len > 0)
	{
#endif

	CYG_ASSERT(buf, "NULL pointer arguments: buf");

#if (defined(TDA7540_USE_EEPROM) || defined(TDA7540_COMPLETE_WITH_EEPROM))
	// address:
	tx_data[0] = addr;

	// lock
	cyg_i2c_transaction_begin(&TunerEEPROM_i2c_device);

	// send the address: start, no stop
	cyg_i2c_transaction_tx(&TunerEEPROM_i2c_device, true, tx_data, 1, false);
	
	// send the data: no start, stop
	cyg_i2c_transaction_tx(&TunerEEPROM_i2c_device, false, buf, len, true);

	// unlock
	cyg_i2c_transaction_end(&TunerEEPROM_i2c_device);
#endif

#ifndef TDA7540_USE_EEPROM
	}
	// Do not write anything without EEPROM.
#endif
}

/**
 * .
 */
int TDA7540_I2CUpdate(cyg_uint8 inStart,cyg_uint8 inStop)
{
	cyg_uint8* buffer;
	int ret = 0;

	TDA7540_GetBuffer(&buffer);
	CYG_ASSERT(inStart<=inStop, "");
	CYG_ASSERT(inStop<=TDA7540_STD_SPIKE, "");
	CYG_ASSERT(inStart>=TDA7540_CHARGE_PUMP, "");
	
	ret = TDA7540_Write(inStart, &(TDA7540Buffer[inStart]), inStop-inStart+1);
	TDA7540_ReleaseBuffer();
	TDA7540_DEBUG_DISPLAY_REG(inStart, inStop, 2);

	return ret;
}

/**
 * .
 */
void TDA7540_LoadEEPROM(void)
{
	cyg_uint8* buffer;

	// Load EEPROM data.
	TDA7540_GetBuffer(&buffer);
	TDA7540_ReadEEPROM(10, buffer, TDA7540_BUFFER_SIZE);
	TDA7540_ReleaseBuffer();

	TDA7540_GetTV2Buffer(&buffer);
	TDA7540_ReadEEPROM(50, buffer, TDA7540_TV2_BUFFER_SIZE);
	TDA7540_ReleaseBuffer();

	TDA7540_DEBUG_DISPLAY_EEPROM(0, TDA7540_BUFFER_SIZE+TDA7540_TV2_BUFFER_SIZE-1, 0);
}

/**
 * .
 */
cyg_bool TDA7540_CheckEEPROM(void)
{
	cyg_bool	retval = 0;
	cyg_bool	ack = false;
	cyg_uint8	eepromcheck = 0;

	ack = TDA7540_ReadEEPROM(0, &eepromcheck, 1);

	retval = (eepromcheck == 0x0C) && ack;
	TDA7540_DEBUG("EEPROM check value = %x\n", eepromcheck);

	return retval;
}

//////////////////////////////////////////////////////////////////////////
//	TDA7540 Registers
//////////////////////////////////////////////////////////////////////////

/**
 * .
 */
void TDA7540_GetBuffer(cyg_uint8** ioTunerBuffer)
{
	cyg_mutex_lock(&TDA7540Mutex);
	(*ioTunerBuffer) = TDA7540Buffer;
}

/**
 * .
 */
void TDA7540_GetBufferReadOnly(const cyg_uint8** ioTunerBuffer)
{
	(*ioTunerBuffer) = TDA7540Buffer;
}

/**
 * .
 */
void TDA7540_GetTV2BufferReadOnly(cyg_uint8** ioTunerTV2Buffer)
{
	(*ioTunerTV2Buffer) = TDA7540TV2Buffer;
}

/**
 * .
 */
void TDA7540_GetTV2Buffer(cyg_uint8** ioTunerTV2Buffer)
{
	cyg_mutex_lock(&TDA7540Mutex);
	(*ioTunerTV2Buffer) = TDA7540TV2Buffer;
}

/**
 * .
 */
void TDA7540_ReleaseBuffer(void)
{
	cyg_mutex_unlock(&TDA7540Mutex);
}

/**
 * .
 */
cyg_int8 TDA7540_GetTV2Value(cyg_uint8 reg)
{
	cyg_uint8* buffer;
	cyg_int8 tv2;

	TDA7540_GetTV2BufferReadOnly(&buffer);

	if ((buffer[reg] & 0x80) == 0x80)
	{
		tv2 = buffer[reg] & 0x7F;
	}
	else
	{
		tv2 = 0x00 - (buffer[reg] & 0x7F);
	}
	return tv2;
}

/**
 * .
 */
void TDA7540_SetRegister(cyg_uint8 reg, cyg_uint8 mask, cyg_uint8 val, TDA7540_UPDATE update)
{
	cyg_uint8* buffer;

	TDA7540_GetBuffer(&buffer);
	TDA7540_SETVAL(buffer[reg], val, mask);
	TDA7540_ReleaseBuffer();
	if (update == TDA7540_WRITE_REG)
	{
		TDA7540_I2CUpdate(reg, reg);
	}
	else
	{
		TDA7540_DEBUG_DISPLAY_REG(reg, reg, 0);
	}
}

/**
 * .
 */
cyg_uint8 TDA7540_GetRegister(cyg_uint8 reg, cyg_uint8 mask)
{
	cyg_uint8* buffer;
	cyg_uint8 value;

	TDA7540_GetBuffer(&buffer);
	TDA7540_GETVAL(buffer[reg], value, mask);
	TDA7540_ReleaseBuffer();
	TDA7540_DEBUG_DISPLAY_REG(reg, reg, 1);

	return value;
}

//////////////////////////////////////////////////////////////////////////[0]

/**
 * .
 */
void TDA7540_SetCPCurrent(TDA7540_FM_HICURRENT locur, TDA7540_FM_HICURRENT hicur, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_CHARGE_PUMP, (cyg_uint8)TDA7540_MASK_CPUMP_LO_CUR, (cyg_uint8)locur, update);
	TDA7540_SetRegister((cyg_uint8)TDA7540_CHARGE_PUMP, (cyg_uint8)TDA7540_MASK_CPUMP_HI_CUR, (cyg_uint8)hicur, update);
}

/**
 * .
 */
void TDA7540_SetCPCurrentMode(TDA7540_CURRENT_MODE mode, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_CHARGE_PUMP, (cyg_uint8)TDA7540_MASK_CPUMP_CURMODE, (cyg_uint8)mode, update);
}

/**
 * .
 */
void TDA7540_SetCPFMSeek(TDA7540_FM_SEEK_STATE state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_CHARGE_PUMP, (cyg_uint8)TDA7540_MASK_CPUMP_FMSEEK, (cyg_uint8)state, update);
}

//////////////////////////////////////////////////////////////////////////[1][2]

/**
 * .
 */
void TDA7540_SetCounter(TDA7540_BAND inBand, cyg_uint32 inFrequency, TDA7540_UPDATE update)
{
	cyg_uint16 pllcounter = 0;
	cyg_uint16 lo = 0;
	cyg_uint16 hi = 0;

	switch (inBand)
	{
	case TDA7540_BAND_FM_EUROPE:
	case TDA7540_BAND_FM_AUSTRALIA:
	case TDA7540_BAND_FM_MIDLEEAST:
	case TDA7540_BAND_FM_AMERICA1:
	case TDA7540_BAND_FM_AMERICA2:
	case TDA7540_BAND_FM_AMERICA3:
	case TDA7540_BAND_FM_AMERICA4:
	case TDA7540_BAND_FM_OIRT:
		pllcounter = ((inFrequency + 10700) /50) - 32; // (Fd + 10.7MHz)/50KHz-32 
	break;
	case TDA7540_BAND_FM_JAPAN:
		pllcounter = ((inFrequency - 10700) /50)  -32;
	break;

	case TDA7540_BAND_MW_EUROPE:
	case TDA7540_BAND_MW_AUSTRALIA:
	case TDA7540_BAND_MW_MIDLEEAST:
	case TDA7540_BAND_MW_JAPAN:
	case TDA7540_BAND_MW_AMERICA1:
	case TDA7540_BAND_MW_AMERICA2:
	case TDA7540_BAND_MW_AMERICA3:
	case TDA7540_BAND_MW_AMERICA4:
	case TDA7540_BAND_LW_EUROPE:
	case TDA7540_BAND_SW2:
		pllcounter = (inFrequency + 10700) -32;
	break;
	case TDA7540_BAND_SW1:
	case TDA7540_BAND_SW3:
		pllcounter = ((inFrequency + 10700)*2) -32;
	break;
	case TDA7540_BAND_SW4:
		pllcounter = (((inFrequency + 10700)*2)/3) -32;
	break;
	case TDA7540_BAND_UNDEFINED:
	default:
		//
		// Unknown band...
		//
	break;
	}	

	lo = ((unsigned char*)&pllcounter)[0];
	hi = ((unsigned char*)&pllcounter)[1];

	TDA7540_SetRegister((cyg_uint8)TDA7540_PLL_PC_LOW, (cyg_uint8)TDA7540_MASK_PLLCOUNTER_LO, (cyg_uint8)lo, update);
	TDA7540_SetRegister((cyg_uint8)TDA7540_PLL_PC_HIGH, (cyg_uint8)TDA7540_MASK_PLLCOUNTER_HI, (cyg_uint8)hi, update);
}

//////////////////////////////////////////////////////////////////////////[3]

/**
 * .
 */
void TDA7540_SetTV1Value(cyg_uint8 tv1, TDA7540_UPDATE update)
{
	cyg_uint8 tv1reg;

	if ((tv1 & 0x80) == 0x80)
	{
		tv1reg = (0x00 - tv1) & 0x7F;
	}
	else
	{
		tv1reg = tv1 | 0x80;
	}

	TDA7540_SetRegister((cyg_uint8)TDA7540_TV2, (cyg_uint8)TDA7540_MASK_TV2, (cyg_uint8)tv1reg, update);
}

//////////////////////////////////////////////////////////////////////////[4]

/**
 * .
 */
void TDA7540_SetTV2Value(cyg_uint8 tv2, TDA7540_UPDATE update)
{
	cyg_uint8 tv2reg;

	if ((tv2 & 0x80) == 0x80)
	{
		tv2reg = (0x00 - tv2) & 0x7F;
	}
	else
	{
		tv2reg = tv2 | 0x80;
	}

	TDA7540_SetRegister((cyg_uint8)TDA7540_TV2, (cyg_uint8)TDA7540_MASK_TV2, (cyg_uint8)tv2reg, update);
}

//////////////////////////////////////////////////////////////////////////[5]

/**
 * .
 */
void TDA7540_SetIFCLockDetector(TDA7540_LOCKSTATE state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_IFC_CONTROL1, (cyg_uint8)TDA7540_MASK_IFCC_LOCKSTATE, (cyg_uint8)state, update);
}

/**
 * .
 */
void TDA7540_SetIFCMuteAndAGC(TDA7540_MUTEAGC muteagc, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_IFC_CONTROL1, (cyg_uint8)TDA7540_MASK_IFCC_MUTEAGC, (cyg_uint8)muteagc, update);
}

/**
 * .
 */
void TDA7540_SetIFCMode(TDA7540_IFCMODE ifcmode, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_IFC_CONTROL1, (cyg_uint8)TDA7540_MASK_IFCC_IFCMODE, (cyg_uint8)ifcmode, update);
}

/**
 * .
 */
void TDA7540_SetIFCState(TDA7540_IFCSTATE state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_IFC_CONTROL1, (cyg_uint8)TDA7540_MASK_IFCC_IFCSTATE, (cyg_uint8)state, update);
}

/**
 * .
 */
void TDA7540_SetIFCSampleTime(TDA7540_TSAMPLE tsample, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_IFC_CONTROL1, (cyg_uint8)TDA7540_MASK_IFCC_TSAMPLE, (cyg_uint8)tsample, update);
}

//////////////////////////////////////////////////////////////////////////[6]

/**
 * .
 */
void TDA7540_SetIFCFCenter(TDA7540_IFC_FCENTER fcenter, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_IFC_CONTROL2, (cyg_uint8)TDA7540_MASK_IFCC_FCENTER, (cyg_uint8)fcenter, update);
}

/**
 * .
 */
void TDA7540_SetIFCErrorWindow(TDA7540_IFC_EW ew, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_IFC_CONTROL2, (cyg_uint8)TDA7540_MASK_IFCC_EW, (cyg_uint8)ew, update);
}

//////////////////////////////////////////////////////////////////////////[7]

/**
 * .
 */
void TDA7540_SetAMControlAGCTimeConst(TDA7540_AMCTRL_AGCTC agctc, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_AM_CONTROL, (cyg_uint8)TDA7540_MASK_AMCTRL_AGCTC, (cyg_uint8)agctc, update);
}

/**
 * .
 */
void TDA7540_SetAMControlPin24(TDA7540_AMCTRL_PIN24 usage, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_AM_CONTROL, (cyg_uint8)TDA7540_MASK_AMCTRL_PIN24, (cyg_uint8)usage, update);
}

/**
 * .
 */
void TDA7540_SetAMControlPrescaler(TDA7540_AMCTRL_PRESCALER prescaler, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_AM_CONTROL, (cyg_uint8)TDA7540_MASK_AMCTRL_PRESCALER, (cyg_uint8)prescaler, update);
}

/**
 * .
 */
void TDA7540_SetAMControlAdditTVOffset(TDA7540_AMCTRL_ADD_OFFSTV offtv, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_AM_CONTROL, (cyg_uint8)TDA7540_MASK_AMCTRL_ADD_OFFSTV, (cyg_uint8)offtv, update);
}

/**
 * .
 */
void TDA7540_SetAMControlPllMode(TDA7540_AMCTRL_TV_PLL_MODE mode, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_AM_CONTROL, (cyg_uint8)TDA7540_MASK_AMCTRL_TVPLL_MODE, (cyg_uint8)mode, update);
}

/**
 * .
 */
void TDA7540_SetAMControlLocalMode(TDA7540_AMCTRL_LOCAL_MODE state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_AM_CONTROL, (cyg_uint8)TDA7540_MASK_AMCTRL_LOCAL_MODE, (cyg_uint8)state, update);
}

//////////////////////////////////////////////////////////////////////////[8]

/**
 * .
 */
void TDA7540_SetQualISSFilterCtrl(TDA7540_QUAL_ISSF_CTRL state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_QUALITY_ISS, (cyg_uint8)TDA7540_MASK_QUAL_ISSF_CTRL, (cyg_uint8)state, update);
}

/**
 * .
 */
void TDA7540_SetQualISSFilterSwitch(TDA7540_QUAL_ISSF_SWITCH state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_QUALITY_ISS, (cyg_uint8)TDA7540_MASK_QUAL_ISSF_SWITCH, (cyg_uint8)state, update);
}

/**
 * .
 */
void TDA7540_SetQualISSFilterFreq(TDA7540_QUAL_ISSF_FREQ freq, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_QUALITY_ISS, (cyg_uint8)TDA7540_MASK_QUAL_ISSF_FREQ, (cyg_uint8)freq, update);
}

/**
 * .
 */
void TDA7540_SetQualISSFilter30KHzWB(TDA7540_QUAL_ISSF_30KHZ state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_QUALITY_ISS, (cyg_uint8)TDA7540_MASK_QUAL_ISSF_30KHZ, (cyg_uint8)state, update);
}

/**
 * .
 */
void TDA7540_SetQualISSFilterCurrent(TDA7540_QUAL_ISSF_CURRENT value, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_QUALITY_ISS, (cyg_uint8)TDA7540_MASK_QUAL_ISSF_CURRENT, (cyg_uint8)value, update);
}

//////////////////////////////////////////////////////////////////////////[9]

//////////////////////////////////////////////////////////////////////////[10]

//////////////////////////////////////////////////////////////////////////[11]

//////////////////////////////////////////////////////////////////////////[12]

//////////////////////////////////////////////////////////////////////////[13]

//////////////////////////////////////////////////////////////////////////[14]

/**
 * .
 */
void TDA7540_SetPllReference(TDA7540_PLL_BAND pllband, TDA7540_PLL_REFERENCE pllref, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_VCO_PLLREF, (cyg_uint8)TDA7540_MASK_PLLREF, (cyg_uint8)pllref, update);
	TDA7540_SetRegister((cyg_uint8)TDA7540_VCO_PLLREF, (cyg_uint8)TDA7540_MASK_PLLBAND, (cyg_uint8)pllband, update);
}

//////////////////////////////////////////////////////////////////////////[15]

//////////////////////////////////////////////////////////////////////////[16]

//////////////////////////////////////////////////////////////////////////[17]

//////////////////////////////////////////////////////////////////////////[18]

/**
 * .
 */
void TDA7540_SetSMeterOffsetSL(TDA7540_SMETER_OFFSET_SL slide, TDA7540_SMETER_OFFSET_SIGN sign, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_SMETER, (cyg_uint8)(TDA7540_MASK_SMETER_OFSL+TDA7540_MASK_SMETER_OFSG), (cyg_uint8)(slide+sign), update);
}

/**
 * .
 */
void TDA7540_SetSMeterOverdevSlope(TDA7540_SMETER_OVERDEV_SLOPE slope, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_SMETER, (cyg_uint8)TDA7540_MASK_SMETER_OD_SLOP, (cyg_uint8)slope, update);
}

/**
 * .
 */
void TDA7540_SetSMeterOverdevCMax(TDA7540_SMETER_OVERDEV_CMAX cmax, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_SMETER, (cyg_uint8)TDA7540_MASK_SMETER_OD_CMAX, (cyg_uint8)cmax, update);
}

/**
 * .
 */
void TDA7540_SetSMeterOverdev(TDA7540_SMETER_OVERDEV state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_SMETER, (cyg_uint8)TDA7540_MASK_SMETER_OD, (cyg_uint8)state, update);
}

//////////////////////////////////////////////////////////////////////////[23]

/**
 * .
 */
void TDA7540_SetSwCtrlSStopRef(TDA7540_SWCTRL_SSTOP ref, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_SSTOP, (cyg_uint8)TDA7540_MASK_SWCTRL_SSTOP, (cyg_uint8)ref, update);
}

/**
 * .
 */
void TDA7540_SetSwCtrlOutput(TDA7540_SWCTRL_OUTPUT_SEL output, TDA7540_SWCTRL_SOUTPUT state, TDA7540_UPDATE update)
{
	cyg_uint8 smask = TDA7540_MASK_SWCTRL_OUTPUT;
	cyg_uint8 svalue = state;

	if (output == TDA7540_SWCTRL_OUTPUT_SEL_S2)
	{
		smask = (smask<<2)&&0x08;
		svalue = (svalue<<2)&&0x08;
	}

	TDA7540_SetRegister((cyg_uint8)TDA7540_SSTOP, (cyg_uint8)TDA7540_MASK_SWCTRL_OUTPUT, (cyg_uint8)state, update);
}

/**
 * .
 */
void TDA7540_SetSwCtrlOutputVoltage(TDA7540_SWCTRL_OUTPUT_SEL output, TDA7540_SWCTRL_SOUTPUT_VOLT mode, TDA7540_UPDATE update)
{
	cyg_uint8 smask = TDA7540_MASK_SWCTRL_OUT_LVL;
	cyg_uint8 svalue = mode;

	if (output == TDA7540_SWCTRL_OUTPUT_SEL_S2)
	{
		smask = (smask<<2)&&0x04;
		svalue = (svalue<<2)&&0x04;
	}

	TDA7540_SetRegister((cyg_uint8)TDA7540_SSTOP, (cyg_uint8)TDA7540_MASK_SWCTRL_OUT_LVL, (cyg_uint8)mode, update);
}

//////////////////////////////////////////////////////////////////////////[24]

//////////////////////////////////////////////////////////////////////////[25]

/**
 * .
 */
void TDA7540_MuteStereoDecoder(TDA7540_STD_MUTE state, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_STD2_IN_GAIN, (cyg_uint8)TDA7540_MASK_STD_MUTE, (cyg_uint8)state, update);
}

/**
 * .
 */
void TDA7540_SetDeEmphasis(TDA7540_STD_DEEMPHASIS value, TDA7540_UPDATE update)
{
	TDA7540_SetRegister((cyg_uint8)TDA7540_STD2_IN_GAIN, (cyg_uint8)TDA7540_MASK_STD_DEEMPH, (cyg_uint8)value, update);
}

//////////////////////////////////////////////////////////////////////////
//	Debug functions
//////////////////////////////////////////////////////////////////////////

/**
 * .
 */
void TDA7540_DebugDisplayTunerBuffer(cyg_uint16 istart, cyg_uint16 istop, int mode)
{
	const cyg_uint8* buffer;
	int i;
	int j;
	char bin[9];

	CYG_ASSERT(istart <= istop, "");
	CYG_ASSERT(istart < TDA7540_BUFFER_SIZE+TDA7540_TV2_BUFFER_SIZE, "");
	CYG_ASSERT(istop < TDA7540_BUFFER_SIZE+TDA7540_TV2_BUFFER_SIZE, "");

	TDA7540_GetBufferReadOnly(&buffer);
	for (i=istart; i<=istop; i++)
	{
		if (i<TDA7540_BUFFER_SIZE)
		{
			for (j=0; j<8; j++)
			{
				if (TDA7540Buffer[i] & (0x01 << (7-j)))
				{
					bin[j] = '1';
				}
				else
				{
					bin[j] = '0';
				}
			}
			bin[8] = '\0';
			TDA7540_DEBUG("[%d] = 0x%02x |%s| ", i, TDA7540Buffer[i], bin);
		}
		else
		{
			for (j=0; j<8; j++)
			{
				if (TDA7540TV2Buffer[i-TDA7540_BUFFER_SIZE] & (0x01 << (7-j)))
				{
					bin[j] = '1';
				}
				else
				{
					bin[j] = '0';
				}
			}
			bin[8] = '\0';
			TDA7540_DEBUG("TV2[%d] = 0x%02x |%s| ", i, TDA7540TV2Buffer[i-TDA7540_BUFFER_SIZE], bin);
		}

		switch(mode)
		{
		case 0:
			TDA7540_DEBUG("\n");
		break;
		case 1:
			TDA7540_DEBUG("R\n");
		break;
		case 2:
			TDA7540_DEBUG("W\n");
		break;
		default:
			TDA7540_DEBUG("\n");
		break;
		}
	}
}
