#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <cyg/kernel/kapi.h>
#include <cyg/io/io.h>
#include <cyg/hal/hal_arch.h> // CYGNUM_HAL_STACK_SIZE_TYPICAL
#include <cyg/io/serialio.h>
#include <cyg/hal/hal_diag.h>

#if defined(CYGHWR_HAL_CUSTOM_CONFIG_P5DEV)
#     include <cyg/hal/p5dev.h> // SELECT_PIN
#endif

#define STACKSIZE CYGNUM_HAL_STACK_SIZE_TYPICAL
#define NTHREADS 3
#define PRIORITY 4

#include "config.h"

#define DEBUG

#define TTY0 0
#define TTY1 1
#define TTY2 2
#define TTY3 3

static char * tty_tab[4] = {
		"/dev/ser0",
		"/dev/ser1",
		"/dev/ser2",
		"/dev/ser3"
};

#define TEST_STRING "HELLO WORLD\n"

static cyg_handle_t thread[NTHREADS];
static cyg_thread thread_obj[NTHREADS];
static char stack[NTHREADS][STACKSIZE];
static unsigned char buffer[NTHREADS][BUFFER_SIZE+2]; // buffer of data + 2 bytes of CRC

#define U16 cyg_uint16


/*
 * Fletcher's checksum
 * @param data:  Pointer to the data to be summed
 * @param len:    Length in 16-bit words
 * @return 16 bits checksum
 */
cyg_uint16 checksum(cyg_uint16 *data, cyg_uint16 len)
{
	cyg_uint32 sum1 = 0xffff, sum2 = 0xffff;

	while (len) {
		unsigned int tlen = len > 360 ? 360 : len;
		len -= tlen;
		do {
			sum1 += *data++;
			sum2 += sum1;
		} while (--tlen);
		sum1 = (sum1 & 0xffff) + (sum1 >> 16);
		sum2 = (sum2 & 0xffff) + (sum2 >> 16);
	}
	/* Second reduction step to reduce sums to 16 bits */
	sum1 = (sum1 & 0xffff) + (sum1 >> 16);
	sum2 = (sum2 & 0xffff) + (sum2 >> 16);
	return sum2 << 16 | sum1;
}

void test_tty(cyg_addrword_t data)
{
	cyg_io_handle_t handle;
	Cyg_ErrNo err;
	cyg_uint32 len;
	cyg_uint32 buf[1];
	char * tty = tty_tab[data];
	int i;
	int nb_try = 0;
	int read_size;
	U16 crc;
        char tmp_buf[512];

	err = cyg_io_lookup(tty, &handle );

	if (err != ENOERR) {
		diag_printf("cyg_io_lookup error\n");
		while(1);
		return;
	}

	// read et write bloquant
	buf[0] = 1;
	len = sizeof(buf[0]);
	cyg_io_set_config(handle, CYG_IO_SET_CONFIG_READ_BLOCKING, buf, &len);
	buf[0] = 1;
	len = sizeof(buf[0]);
	cyg_io_set_config(handle, CYG_IO_SET_CONFIG_WRITE_BLOCKING, buf, &len);

	while (1) {
#ifdef SEND_DATA_TO_HOST
		// send random buffer with CRC
		for (i = 0; i < BUFFER_SIZE; i++)
				buffer[data][i] = rand();
		*(cyg_uint16 *) &buffer[data][BUFFER_SIZE]
				= checksum((U16*) buffer[data], BUFFER_SIZE/2);

		len = (BUFFER_SIZE+2);
		cyg_io_write(handle, buffer[data], &len);
#endif

#ifdef RECEIVE_DATA_ON_P5_BOARD
		// read byte from PC
		read_size = 0;
		while (read_size != BUFFER_SIZE+2) {
			len = BUFFER_SIZE+2-read_size;
#ifdef DEBUG
			diag_printf("\n######## read %d ########\n", nb_try);
#endif
			cyg_io_read(handle, buffer[data]+read_size, &len);
#ifdef DEBUG
			diag_printf("######## %d/%d ########\n", read_size, BUFFER_SIZE+2);
#endif
			read_size += len;
		}

#if 0
		// verify the checksum
		crc = checksum((U16 *) buffer[data], BUFFER_SIZE/2);
		if (crc != *(U16*) &buffer[data][BUFFER_SIZE]) {
			// checksum error
			diag_printf(">>>>>>>>>> CRC ERROR <<<<<<<<<<\n");
			while(1);
		}
		diag_printf("CRC OK\n");
#endif
#endif
		nb_try++;
	}
}




void cyg_user_start(void)
{
#if defined(CYGHWR_HAL_CUSTOM_CONFIG_P5DEV)
	SELECT_PIN(UART2_TX);
	SELECT_PIN(UART2_RX);

	SELECT_PIN(UART0_RTS);
	SELECT_PIN(UART0_CTS);
	SELECT_PIN(UART1_RTS);
	SELECT_PIN(UART1_CTS);
#endif
        diag_init();	
#if 0
	cyg_thread_create(PRIORITY, test_tty, (cyg_addrword_t) TTY0, "test_tty0",
					(void *)stack[0], STACKSIZE, &thread[0], &thread_obj[0]);
	cyg_thread_resume(thread[0]);
#endif	
        cyg_thread_create(PRIORITY, test_tty, (cyg_addrword_t) TTY2, "test_tty2",
					(void *)stack[0], STACKSIZE, &thread[0], &thread_obj[0]);
	cyg_thread_resume(thread[0]);
	/*
	cyg_thread_create(PRIORITY, test_tty, (cyg_addrword_t) TTY1, "test_tty1",
					(void *)stack[1], STACKSIZE, &thread[1], &thread_obj[1]);
	cyg_thread_resume(thread[1]);
	
	
	cyg_thread_create(PRIORITY, test_tty, (cyg_addrword_t) TTY2, "test_tty2",
					(void *)stack[2], STACKSIZE, &thread[2], &thread_obj[2]);
	cyg_thread_resume(thread[2]);*/
}
