//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * @file   lpo_boot.c
 * @brief  Parrot5 LPO boot time routines
 *
 * @author ivan.djelic@parrot.com
 * @date   2008-03-27
 *
 * This file is licensed under the eCos Public License. A copy of this license
 * can be obtained from:
 * http://ecos.sourceware.org/license-overview.html
 *
 * The Original Code is eCos - Embedded Configurable Operating System,
 *
 * Portions created by Parrot S.A. are
 * Copyright (C) 2004-2008 Parrot S.A.
 */

#include <task_priorities.h>
#include <pkgconf/hal.h>                // Platform specific configury
#include <pkgconf/system.h>
#include CYGBLD_HAL_PLATFORM_H
#include CYGHWR_HAL_CUSTOM_CONFIG_H

#include <cyg/hal/hal_io.h>             // IO macros
#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_intr.h>           // interrupt enable/disable
#include <cyg/kernel/kapi.h>

#include <cyg/hal/p5dev.h>              // platform definitions
#include <cyg/hal/lpo.h>
#include <cyg/hal/hal_platform_ints.h>
#include <cyg/hal/hal_diag.h>
#include "parrot5_lpo.h"

#if defined(CYGHWR_HAL_CUSTOM_CONFIG_MOTOS) || \
    defined(CYGHWR_HAL_CUSTOM_CONFIG_MYNOS2) || \
    defined(CYGHWR_HAL_CUSTOM_CONFIG_TONGA)

/**
 * Switch the Parrot5 from Stage1 (after RTC wake-up and LED setting) to
 * STANDBY mode, the LPO control register is configured for external wake up
 * and alarm wake up
 *
 * @param rtc_alarm Program LPO for wake-up at RTC date 'rtc_alarm'.
 *                  Passing 0 will disable RTC alarm wake-up.
 */
void lpo_standby_enable(cyg_uint32 rtc_alarm)
{
    cyg_uint32 reg;

    // Acknowledge all events
    LPO_WRITE(_LPO_ITACK, LPO_ITACK_RTCA|LPO_ITACK_EMCY|LPO_ITACK_WKUP);

    LPO_WRITE(_LPO_RTCA, rtc_alarm);

    // Program STANDBY exit conditions
    LPO_WRITE(_LPO_CONTROL, LPO_CTRL_WKUP_EN | LPO_CTRL_ALARM_EN);

    // Synchronize with 32 kHz block
    do {
        HAL_READ_UINT32(PARROT5_LPO+_LPO_CONTROL, reg);
    }
    while (reg != 3);

    // These magic values are documented in Parrot5 User Manual � 5.3.3
    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_SDBYMODE, 0x2b56924d);
    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_SDBYMODE, 0x4ad365b2);

    // Now wait...
    while (1) ;
}
#endif  /* CYGHWR_HAL_CUSTOM_CONFIG_MOTOS || CYGHWR_HAL_CUSTOM_CONFIG_MYNOS2 || CYGHWR_HAL_CUSTOM_CONFIG_TONGA */
