#ifndef CYGONCE_HAL_CPU_TYPES_H
#define CYGONCE_HAL_CPU_TYPES_H

/**
 * Brief: some usefull function for knowing on which CPU we are running
 *
 *
 * We left the name of function in uppercase for not changing all the code
 */

/*
 * Function for getting total configured RAM
 *
 * Return RAM size in bytes
 */
inline unsigned int cyg_hal_ram_size(void);

/*
 * Function for getting configured RAM name
 *
 * Return a string
 */
inline const char *cyg_hal_ram_name(void);

/*
 * Function for getting configured RAM type DDR/SDR
 *
 * Return an int
 */
inline const int cyg_hal_ram_issdr(void);

#include <pkgconf/system.h>

#ifndef __ASSEMBLER__

typedef enum cpu_types {
    PARROT_CHIP_P5,
    PARROT_CHIP_P5P,
    PARROT_CHIP_P6,
    PARROT_CHIP_P6I,
    PARROT_CHIP_P7,
    TI_CHIP_OMAP3630,
    CPU_TYPES_UNKNOWN,
} cpu_types;

#if defined(CYGPKG_HAL_ARM_ARM9_P5DEV)
#include <cyg/hal/p5dev.h>
static inline int parrot_chipid(void)
{
    return (*(volatile unsigned int *)(PARROT5_SYS + _SYS_CHIPID));
}

static inline int PARROT_CHIP_IS_P5(void)
{
    return parrot_chipid() == SYS_CHIPID_P5;
}
static inline int PARROT_CHIP_IS_P5P(void)
{
    return parrot_chipid() == SYS_CHIPID_P5P;
}

/*According to P5P_R4C user manual
 *To identify the R4C version, follow these steps:
 * - Enable DCT (CAMIF) clock with SYSC Clock Enable register
 * - Read DCT signature register at address 0xD00B0034
 * - If the read value is 0x0, the chip is P5+R4C
 * - If the read value is 0x30304344, the chip is P5+
 */
static inline int PARROT_CHIP_P5P_IS_R3C(void)
{
    *((volatile unsigned int*)(PARROT5_SYS+_SYS_CEN)) = SYS_CEN_CAMIFCLK;
    return ((*(volatile unsigned int *)(0xD00B0034)) == 0x30304344) ? 1 : 0;
}

static inline int PARROT_CHIP_P5P_IS_R4C(void)
{
    *((volatile unsigned int*)(PARROT5_SYS+_SYS_CEN)) = SYS_CEN_CAMIFCLK;
    return ((*(volatile unsigned int *)(0xD00B0034)) == 0) ? 1 : 0;
}

static inline int PARROT_CHIP_IS_P6(void) { return 0; }
static inline int PARROT_CHIP_IS_P6I(void) { return 0; }
static inline int PARROT_CHIP_IS_P7(void) { return 0; }
static inline int TI_CHIP_IS_OMAP3630(void) { return 0; }

static inline int get_cpu_types(void)
{
    if (PARROT_CHIP_IS_P5())
        return PARROT_CHIP_P5;
    else if (PARROT_CHIP_IS_P5P())
        return PARROT_CHIP_P5P;
    else
        return CPU_TYPES_UNKNOWN;
}

#elif defined(CYGPKG_HAL_ARM_ARM9_P6DEV)
#include <cyg/hal/p6dev.h>
static inline int parrot_chipid(void)
{
    return (*(volatile unsigned int *)(PARROT6_SYS+_P6_SYS_CHIPID));
}

static inline int PARROT_CHIP_IS_P5(void) { return 0; }
static inline int PARROT_CHIP_IS_P5P(void) { return 0; }
static inline int PARROT_CHIP_P5P_IS_R3C(void) { return 0; }
static inline int PARROT_CHIP_P5P_IS_R4C(void) { return 0; }
static inline int PARROT_CHIP_IS_P6(void)
{
    return parrot_chipid() == P6_SYS_CHIPID_P6;
}

static inline int PARROT_CHIP_IS_P6I(void)
{
    return parrot_chipid() == P6_SYS_CHIPID_P6I;
}

static inline int PARROT_CHIP_IS_P7(void) { return 0; }
static inline int TI_CHIP_IS_OMAP3630(void) { return 0; }

static inline int get_cpu_types(void)
{
    if (PARROT_CHIP_IS_P6())
        return PARROT_CHIP_P6;
    else if (PARROT_CHIP_IS_P6I())
        return PARROT_CHIP_P6I;
    else
        return CPU_TYPES_UNKNOWN;
}

#elif defined(CYGPKG_HAL_CORTEXA_OMAP_3630)
#include <cyg/hal/omap3630.h>
static inline int PARROT_CHIP_IS_P5(void) { return 0; }
static inline int PARROT_CHIP_IS_P5P(void) { return 0; }
static inline int PARROT_CHIP_P5P_IS_R3C(void) { return 0; }
static inline int PARROT_CHIP_P5P_IS_R4C(void) { return 0; }
static inline int PARROT_CHIP_IS_P6(void) { return 0; }
static inline int PARROT_CHIP_IS_P6I(void) { return 0; }
static inline int PARROT_CHIP_IS_P7(void) { return 0; }
static inline int TI_CHIP_IS_OMAP3630(void) { return 1; }

static inline int get_cpu_types(void) { return TI_CHIP_OMAP3630; }

#elif defined(CYGPKG_HAL_CORTEXA_P7)
#include <cyg/hal/p7.h>
static inline int PARROT_CHIP_IS_P5(void) { return 0; }
static inline int PARROT_CHIP_IS_P5P(void) { return 0; }
static inline int PARROT_CHIP_P5P_IS_R3C(void) { return 0; }
static inline int PARROT_CHIP_P5P_IS_R4C(void) { return 0; }
static inline int PARROT_CHIP_IS_P6(void) { return 0; }
static inline int PARROT_CHIP_IS_P6I(void) { return 0; }
static inline int PARROT_CHIP_IS_P7(void) { return 1; }
static inline int TI_CHIP_IS_OMAP3630(void) { return 0; }

static inline int get_cpu_types(void) { return PARROT_CHIP_P7; }
#endif

#endif /* __ASSEMBLER__ */
#endif /* CYGONCE_HAL_CPU_TYPES_H */
