//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/*=============================================================================
//
//      hal_diag.c
//
//      HAL diagnostic output code
//
//=============================================================================
//
// Author(s):   ivan.djelic@parrot.fr
// Date:        2005-11-18
// Purpose:     HAL diagnostic output
// Description: Implementations of HAL diagnostic output support.
//
//===========================================================================*/

#include <pkgconf/hal.h>
#include CYGBLD_HAL_VARIANT_H           // Variant specific configuration
#include CYGBLD_HAL_PLATFORM_H          // Platform specific configuration
#include <pkgconf/hal_arm_arm9_p6dev.h>

#include <cyg/infra/cyg_type.h>         // base types
#include <cyg/infra/cyg_trac.h>         // tracing macros
#include <cyg/infra/cyg_ass.h>          // assertion macros

#include <cyg/hal/hal_arch.h>           // basic machine info
#include <cyg/hal/hal_intr.h>           // interrupt macros
#include <cyg/hal/hal_io.h>             // IO macros
#include <cyg/hal/hal_diag.h>
#include <cyg/hal/drv_api.h>
#include <cyg/hal/hal_if.h>             // interface API
#include <cyg/hal/hal_misc.h>           // Helper functions
#include <cyg/hal/p6dev.h>         // platform definitions

#ifdef CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC
//-----------------------------------------------------------------------------
// simple DCC interface

#define DCC_OUTPUT_BUSY 2
#define DCC_INPUT_READY 1

#define DCC_READ(_x)  asm volatile ("mrc p14,0,%0,c1,c0\n" :  "=r" (_x))
#define DCC_WRITE(_x) asm volatile ("mcr p14,0,%0,c1,c0\n" : : "r" (_x))
#define DCC_POLL(_x)  asm volatile ("mrc p14,0,%0,c0,c0\n" :  "=r" (_x))

static void dcc_putc(unsigned char c)
{
    cyg_uint32 poll;

    // wait until DCC interface is ready
    do {
        DCC_POLL(poll);
    } while (poll & DCC_OUTPUT_BUSY);

    DCC_WRITE(c);
}

int dcc_getc(void)
{
    cyg_uint32 poll, val;

    do {
        DCC_POLL(poll);
    } while (!(poll & DCC_INPUT_READY));

    DCC_READ(val);

    return (int)(val & 0xff);
}

int dcc_getc_nonblock(void)
{
    int ret = -1;
    cyg_uint32 poll, val;

    DCC_POLL(poll);

    if (poll & DCC_INPUT_READY) {
        DCC_READ(val);
        ret = (int)(val & 0xff);
    }

    return ret;
}

//-----------------------------------------------------------------------------
#endif /* CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC */

typedef struct {
    cyg_uint8  *base;
    const char *clockname;
    cyg_int32   msec_timeout;
    int         isr_vector;
    cyg_int32   baud_rate;
    int         int_state;
} channel_data_t;

static channel_data_t p6dev_ser_channels[] = {
    {
        (cyg_uint8 *)PARROT6_UART0,
        "uart0",
        0,
        CYGNUM_HAL_INTERRUPT_UART0,
        CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD
    },
    {
        (cyg_uint8 *)PARROT6_UART1,
        "uart1",
        0,
        CYGNUM_HAL_INTERRUPT_UART1,
        CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD
    },
    {
        (cyg_uint8 *)PARROT6_UART2,
        "uart2",
        0,
        CYGNUM_HAL_INTERRUPT_UART2,
        CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD
    },
    {
        (cyg_uint8 *)PARROT6_UART3,
        "uart3",
        0,
        CYGNUM_HAL_INTERRUPT_UART3,
        CYGNUM_HAL_VIRTUAL_VECTOR_CONSOLE_CHANNEL_BAUD
    },
};

//-----------------------------------------------------------------------------

void cyg_hal_plf_serial_init_channel(void *__ch_data)
{
    const struct pclock *clk;
    cyg_uint32 baud_divisor, lcr;
    channel_data_t *p6_chan = ((channel_data_t*)__ch_data);
    cyg_uint8 *base = p6_chan->base;

    // start UART clock in P6 system controller
    clk = pclock_get(p6_chan->clockname);
    if (clk) {
        pclock_enable(clk);

        // first stop and reset UART
        HAL_WRITE_UINT32(base+_UART_SPD, 0);

        // set baudrate
        baud_divisor = pclock_get_rate(clk)/p6_chan->baud_rate;

        // Set 8 bits, 1 stop bit, no parity, enable FIFOs
        lcr =
            UART_LCR_EIGHTBITS|
            UART_LCR_TXFIFO|
            UART_LCR_RXFIFO|
            UART_LCR_FIFO_THRESHOLD_16;

        // write configuration
        HAL_WRITE_UINT32(base+_UART_LCR, lcr);
        // enable UART
        HAL_WRITE_UINT32(base+_UART_SPD, baud_divisor);
    }
}

void cyg_hal_plf_serial_putc(void *__ch_data, char c)
{

#ifdef CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC
    dcc_putc(c);
#else
    cyg_uint8 *base = ((channel_data_t*)__ch_data)->base;
    cyg_uint32 status;

    do {
        HAL_READ_UINT32(base+_UART_STATUS, status);
    } while (status & UART_STATUS_TXFILLED);

    HAL_WRITE_UINT32(base+_UART_TRX, c);

#endif /* CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC */
}

static cyg_bool cyg_hal_plf_serial_getc_nonblock(void *__ch_data, cyg_uint8 *ch)
{
#ifdef CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC
    int ret = dcc_getc_nonblock();
    if (ret < 0) {
        return false;
    }
    *ch = ret;
    return true;
#else

    cyg_uint8 *base = ((channel_data_t*)__ch_data)->base;
    cyg_uint32 status;
    cyg_uint32 c;

    HAL_READ_UINT32(base+_UART_STATUS, status);

    if (!(status & UART_STATUS_RXFILLED)) {
        return false;
    }
    HAL_READ_UINT32(base+_UART_TRX, c);
    // check for errors
    if (c & ~0xff) {
        return false;
    }
    *ch = c;
    return true;

#endif /* CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC */
}

cyg_uint8 cyg_hal_plf_serial_getc(void *__ch_data)
{
#ifdef CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC
    return dcc_getc();
#else
    cyg_uint8 ch;

    while (!cyg_hal_plf_serial_getc_nonblock(__ch_data, &ch));
    return ch;
#endif /* CYGHWR_HAL_ARM_ARM9_P6DEV_DIAG_TO_DCC */
}

static void cyg_hal_plf_serial_write(void *__ch_data, const cyg_uint8 *__buf,
                                     cyg_uint32 __len)
{
    while(__len-- > 0) {
        cyg_hal_plf_serial_putc(__ch_data, *__buf++);
    }
}

static void cyg_hal_plf_serial_read(void *__ch_data, cyg_uint8 *__buf,
                                    cyg_uint32 __len)
{
    while(__len-- > 0) {
        *__buf++ = cyg_hal_plf_serial_getc(__ch_data);
    }
}

cyg_bool cyg_hal_plf_serial_getc_timeout(void *__ch_data, cyg_uint8 *ch)
{
    int delay_count;
    channel_data_t *chan = (channel_data_t*)__ch_data;
    cyg_bool res;

    delay_count = chan->msec_timeout * 10; // delay in .1 ms steps

    for(;;) {
        res = cyg_hal_plf_serial_getc_nonblock(__ch_data, ch);
        if (res || (0 == delay_count--)) {
            break;
        }
        CYGACC_CALL_IF_DELAY_US(100);
    }
    return res;
}

static int
cyg_hal_plf_serial_control(void *__ch_data, __comm_control_cmd_t __func, ...)
{
    channel_data_t *chan = (channel_data_t*)__ch_data;
    cyg_uint8 *base = ((channel_data_t*)__ch_data)->base;
    int ret = 0;

    switch (__func) {

    case __COMMCTL_IRQ_ENABLE:
        chan -> int_state = 1;
        // enable RX interrupts
        HAL_WRITE_UINT32(base+_UART_IRX, UART_IRX_RIM);
        HAL_INTERRUPT_UNMASK(chan->isr_vector);
        break;

    case __COMMCTL_IRQ_DISABLE:
        ret = chan->int_state;
        chan->int_state = 0;
        HAL_WRITE_UINT32(base+_UART_IRX, 0);
        HAL_INTERRUPT_MASK(chan->isr_vector);
        break;

    case __COMMCTL_GETBAUD:
        ret = chan->baud_rate;
        break;

    case __COMMCTL_SETBAUD:
        {
            cyg_uint32 b;
            va_list ap;

            va_start(ap, __func);
            b = va_arg(ap, cyg_int32);
            va_end(ap);

            chan->baud_rate = b;
            cyg_hal_plf_serial_init_channel(chan);
            ret = 1;
            break;
        }

    case __COMMCTL_DBG_ISR_VECTOR:
        ret = chan->isr_vector;
        break;

    case __COMMCTL_SET_TIMEOUT:
        {
            va_list ap;

            va_start(ap, __func);

            ret = chan->msec_timeout;
            chan->msec_timeout = va_arg(ap, cyg_uint32);

            va_end(ap);
        }
    default:
        break;
    }
    return ret;
}

static int
cyg_hal_plf_serial_isr(void *__ch_data, int *__ctrlc,
                       CYG_ADDRWORD __vector, CYG_ADDRWORD __data)
{
    int res = 0;
    channel_data_t *chan = (channel_data_t *)__ch_data;
    cyg_uint8 *base = ((channel_data_t *)__ch_data)->base;
    cyg_uint32 status;
    cyg_uint32 _c;
    char c;

    cyg_drv_interrupt_acknowledge(chan->isr_vector);
    *__ctrlc = 0;
    HAL_READ_UINT32(base+_UART_STATUS, status);

    if (status & UART_STATUS_RXFILLED) {
        HAL_READ_UINT32(base+_UART_TRX, _c);
        c = (char)_c;
        if(cyg_hal_is_break(&c , 1))
            *__ctrlc = 1;
        res = CYG_ISR_HANDLED;
    }
    return res;
}

static void
cyg_hal_plf_serial_init(void)
{
    hal_virtual_comm_table_t *comm;
    int cur = CYGACC_CALL_IF_SET_CONSOLE_COMM(CYGNUM_CALL_IF_SET_COMM_ID_QUERY_CURRENT);
    int i;

    for (i = 0;  i < CYG_NELEM(p6dev_ser_channels);  i++) {
        // Init channels
        cyg_hal_plf_serial_init_channel(&p6dev_ser_channels[i]);
        // Setup procs in the vector table
        CYGACC_CALL_IF_SET_CONSOLE_COMM(i);
        comm = CYGACC_CALL_IF_CONSOLE_PROCS();
        CYGACC_COMM_IF_CH_DATA_SET(*comm, &p6dev_ser_channels[i]);
        CYGACC_COMM_IF_WRITE_SET(*comm, cyg_hal_plf_serial_write);
        CYGACC_COMM_IF_READ_SET(*comm, cyg_hal_plf_serial_read);
        CYGACC_COMM_IF_PUTC_SET(*comm, cyg_hal_plf_serial_putc);
        CYGACC_COMM_IF_GETC_SET(*comm, cyg_hal_plf_serial_getc);
        CYGACC_COMM_IF_CONTROL_SET(*comm, cyg_hal_plf_serial_control);
        CYGACC_COMM_IF_DBG_ISR_SET(*comm, cyg_hal_plf_serial_isr);
        CYGACC_COMM_IF_GETC_TIMEOUT_SET(*comm, cyg_hal_plf_serial_getc_timeout);
    }

    // Restore original console
    CYGACC_CALL_IF_SET_CONSOLE_COMM(cur);
}

void
cyg_hal_plf_comms_init(void)
{
    static int initialized = 0;

    if (initialized)
        return;

    initialized = 1;

    cyg_hal_plf_serial_init();
}

/*---------------------------------------------------------------------------*/
/* End of hal_diag.c */
