//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  memprot_p5dev.c
 * @brief Parrot5 board memory protection specific setup functions
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-04
 * ============================================================================
 */

#include "mmu.h"
#include "diag.h"

#ifdef CYGPKG_HAL_ARM_ARM9_P5DEV

// get memory layout
#include <pkgconf/system.h>
#include CYGHWR_MEMORY_LAYOUT_H

#include "memprot.h"

/** P5DEV specific initialisation.
 */
void memprot_p5dev_setup(void) {
  // flat map first 4 kB (VSRs in I-TCM) uncacheable
  memprot_map( 0x00000000, 0x00000000, 0x1000, AP_RW_RO, 0, 0 );
  // forbid user access to bottom 1 kB (VSRs)
  memprot_mmu_set_subpage_ap( 0x00000000, AP_RW_NA );
  // flat map rest of TCMs uncacheable
  memprot_map( 0x00001000, 0x00001000, 0xf000, AP_RW_RO, 0, 0 );
  // map INTRAM cached
  memprot_map( 0x00010000, 0x02000000, 0x18000, AP_RW_RO, 1, 0 );

  // map INTRAM uncached (for Baseband DMA buffers)
  memprot_map( 0x00030000, 0x02000000, 0x18000, AP_RW_RW, 0, 0 );

#if CYGMEM_REGION_sdram_SIZE > 0
  // map SDRAM bank uncached
  memprot_map( CYGMEM_REGION_sdram_PHYS, CYGMEM_REGION_sdram_PHYS,
	       CYGMEM_REGION_sdram_SIZE, AP_RW_NA, 0, 0 );
  // remap SDRAM bank with cache and buffer disabled
  memprot_map( CYGMEM_REGION_sdram_SHADOW, CYGMEM_REGION_sdram_PHYS,
	       CYGMEM_REGION_sdram_SIZE, AP_RW_NA, 0, 0 );
#endif

#if CYGMEM_REGION_flash_SIZE > 0
  // FIXME: map FLASH (uncacheable and unbufferable) read-only
  memprot_map( CYGMEM_REGION_flash, CYGMEM_REGION_flash_PHYS,
	       CYGMEM_REGION_flash_SIZE, AP_RW_RO, 0, 0 );
  // remap shadow FLASH (uncacheable and unbufferable)
  memprot_map( CYGMEM_REGION_flash_SHADOW, CYGMEM_REGION_flash_PHYS,
	       CYGMEM_REGION_flash_SIZE, AP_RW_NA, 0, 0 );
#endif

#if CYGMEM_REGION_sram_SIZE > 0
  // map SRAM uncached
  memprot_map( CYGMEM_REGION_sram_PHYS, CYGMEM_REGION_sram_PHYS,
	       CYGMEM_REGION_sram_SIZE, AP_RW_NA, 0, 0 );
  // remap shadow SRAM (uncacheable and unbufferable)
  memprot_map( CYGMEM_REGION_sram_SHADOW, CYGMEM_REGION_sram_PHYS,
	       CYGMEM_REGION_sram_SIZE, AP_RW_NA, 0, 0 );
#endif

  // Virtual CS GPIO
  memprot_map( 0x50000000, 0x50000000, 0x100000, AP_RW_RO, 0, 0 );
  // AMBA peripherals
#if defined(CYGHWR_HAL_CUSTOM_CONFIG_PSS) || \
    defined(CYGHWR_HAL_CUSTOM_CONFIG_BTT)
  memprot_map( 0xc0000000, 0xc0000000, 0x30000, AP_RW_RW, 0, 0 );
  memprot_map( 0xd0000000, 0xd0000000, 0xb0000, AP_RW_RW, 0, 0 );
  memprot_map( 0xe0000000, 0xe0000000, 0x80000, AP_RW_RW, 0, 0 );
#else
  memprot_map( 0xc0000000, 0xc0000000, 0x30000, AP_RW_NA, 0, 0 );
  memprot_map( 0xd0000000, 0xd0000000, 0xb0000, AP_RW_NA, 0, 0 );
  memprot_map( 0xe0000000, 0xe0000000, 0x80000, AP_RW_NA, 0, 0 );
#endif
}

#endif
