//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  memprot_smdk2410.c
 * @brief Samsung SMDK2410/PSS/WVW boards specific setup
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-04
 * ============================================================================
 */

#include "mmu.h"
#include "diag.h"

#ifdef CYGPKG_HAL_ARM_ARM9_SMDK2410

#include <pkgconf/hal_arm_arm9_smdk2410.h>

// get memory layout
#include <pkgconf/system.h>
#include CYGHWR_MEMORY_LAYOUT_H

#include "memprot.h"

/** SMDK2410 specific initialisation.
 */
void memprot_smdk2410_setup(void) {
  // map first 4 KB page (VSRs) not accessible in user context
  memprot_map( 0x00000000, CYGMEM_REGION_ram_PHYS, 0x1000, AP_RW_NA, 1, 0 );
  // don't map RedBoot code + data
  //memprot_map( 0x00008000, V2P(0x00008000), 0x40000-0x8000, AP_NA_NA, 0, 0 );
  // remap RAM flat with cache and buffer disabled
  memprot_map( CYGMEM_REGION_ram_PHYS, CYGMEM_REGION_ram_PHYS,
	       CYGMEM_REGION_ram_SIZE, AP_RW_NA, 0, 0 );
  // flat map SFRs - 512 MB
  memprot_map( 0x48000000, 0x48000000, 0x20000000, AP_RW_NA, 0, 0 );
  // flat map internal 4 KB SRAM (cacheable and bufferable)
  memprot_map( 0x40000000, 0x40000000, 0x1000, AP_RW_NA, 1, 0 );
  // flat map FLASH (cacheable and bufferable) read-only
#ifdef CYGNUM_HAL_ARM_SMDK2410_FLAVOR_SMDK2410
  memprot_map( 0x80000000, 0x00000000, 0x400000, AP_RW_RO, 1, 0 );
#endif
#ifdef CYGNUM_HAL_ARM_SMDK2410_FLAVOR_PSS_DEV01
  memprot_map( 0x80000000, 0x00000000, 0x200000, AP_RW_RO, 1, 0 );
#endif
}

#endif

