//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  mmu.h
 * @brief MMU programming functions header
 *
 * MMU handling routines.
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-20
 * ============================================================================
 */
#ifndef CYGONCE_MEMPROT_MMU_H
#define CYGONCE_MEMPROT_MMU_H

#include <cyg/infra/cyg_type.h>
#include <pkgconf/system.h>

#define MMU_NO_ALLOCATE 0
#define MMU_ALLOCATE    1

//! System access = read-only, User access = read-only (assuming S=0, R=1)
#define AP_RO_RO	0x0
//! System access = read/write, User access = no access
#define AP_RW_NA	0x1
//! System access = read/write, User access = read-only
#define AP_RW_RO	0x2
//! System access = read/write, User access = read/write
#define AP_RW_RW	0x3

#ifdef CYGPKG_HAL_ARM_ARM9_ARM926E
//! Clean and invalidate data cache
#define FLUSH_CACHE()                                           \
        asm volatile (                                          \
            "1:\n\t"                                            \
            "mrc p15,0,r15,c7,c14,3\n\t"                        \
            "bne 1b\n\t"                                        \
            "mov r0,#0\n\t"                                     \
            "mcr p15,0,r0,c7,c10,4\n\t"                         \
            "nop\n\tnop\n\t" : : : "r0" )
#endif
#ifdef CYGPKG_HAL_ARM_ARM9_ARM920T
#define FLUSH_CACHE()                                           \
        asm volatile (                                          \
	    "mov r1,#7 << 5\n\t"                                \
            "1:\n\t"                                            \
	    "orr r3,r1,#63 << 26\n\t"                           \
            "2:\n\t"                                            \
	    "mcr p15,0,r3,c7,c14,2\n\t"                         \
	    "subs r3,r3,#1 << 26\n\t"                           \
	    "bcs 2b\n\t"                                        \
            "subs r1,r1,#1 << 5\n\t"                            \
	    "bcs 1b\n\t"                                        \
	    "mov r1,#0\n\t"                                     \
            "mcr p15,0,r1,c7,c10,4\n\t"                         \
            "nop\n\tnop\n\t" : : : "r1", "r3" )
#endif

typedef CYG_ADDRESS (* arm_mmu_phys_allocate_t)(size_t);
typedef void (* arm_mmu_phys_free_t)(CYG_ADDRESS, size_t);

void memprot_mmu_configure( unsigned int *_ttb,
                            arm_mmu_phys_allocate_t _alloc,
                            arm_mmu_phys_free_t _free,
                            arm_mmu_phys_allocate_t _alloc_pt,
                            arm_mmu_phys_free_t _free_pt );

CYG_ADDRESS memprot_mmu_physical_address( CYG_ADDRESS vaddr );
int memprot_mmu_map( CYG_ADDRESS vaddr, CYG_ADDRESS paddr, size_t size, int ap,
                     int cache, int domain, int allocate, int use_small_pages);
int memprot_mmu_unmap( CYG_ADDRESS vaddr, size_t size, cyg_bool deallocate );
void memprot_mmu_init(void);
void memprot_mmu_enable(unsigned int *_ttb);
void memprot_mmu_disable(void);
void memprot_mmu_dump_memory_map(void);
void memprot_mmu_get_data_abort_info( CYG_ADDRESS *vaddr, int *domain,
                                      char ** msg );
int memprot_mmu_get_domain( CYG_ADDRESS vaddr );
int memprot_mmu_get_ap( CYG_ADDRESS vaddr );
int memprot_mmu_set_subpage_ap( CYG_ADDRESS vaddr, int ap );
cyg_uint32 memprot_mmu_get_DACR(void);
void memprot_mmu_set_DACR(cyg_uint32 dacr, cyg_uint32 mask);
void memprot_mmu_switch_ttb(unsigned int *new_ttb);

#endif // CYGONCE_MEMPROT_MMU_H
