//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * ============================================================================
 * @file  swi_handler.S
 * @brief SWI exception handler
 *
 * Copyright (C) 2004,2005 Parrot S.A.
 *
 * @author     ivan.djelic@parrot.fr
 * @date       2004-05-26
 * ============================================================================
 */

#include "../../hal/arm/arch/current/arm.inc"
#include <pkgconf/memprot.h>

	.code	32
	.text
	.section	".text.swi_handler","ax"
	.global	swi_handler

swi_handler:
	// which mode did we come from ?
	mrs	ip,spsr
	and	ip,ip,#CPSR_MODE_BITS
	cmp	ip,#CPSR_USER_MODE
	beq	user_mode

	// we came from a privileged mode
	ldr	lr,[lr]				// lr = function to call
	movs	pc,lr				// return to pre-SWI mode

user_mode:
	ldr	sp,.cyg_current_thread_table
	ldr	sp,[sp]				// sp = current thread pointer
	ldr	ip,.memprot_data_index
	ldr	ip,[ip]                         // ip = memprot data index
	add	sp,sp,#THREAD_DATA	        // sp = per-thread data array
	ldr	sp,[sp,ip,lsl #2]               // sp = user thread data
	stmia	sp,{r0-r4}			// save some registers
	mov	r3,sp				// get svc_sp
	mrs	r4,spsr
	str	r4,[sp,#4*5]			// save svc_spsr
	mov	r2,lr				// r2 = swi instr address + 4

	// grant client access to all domains
	mrc	p15,0,r0,c3,c0,0
	str	r0,[r3,#4*7]			// save DACR
	ldr	r1,=CYGOPT_MEMPROT_DACR_MASK
	bic	r0,r0,r1			// r0 = DACR & ~M
	ldr	r1,=(CYGOPT_MEMPROT_DACR_MASK & 0x55555555)
	orr	r0,r0,r1
	mcr	p15,0,r0,c3,c0,0		// write back DACR

	// switch to SYSTEM mode with interrupts enabled
	mrs	r0,cpsr
	bic	r0,r0,#CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE|CPSR_MODE_BITS
	orr	r0,r0,#CPSR_SYSTEM_MODE
	msr	cpsr,r0

	str	lr,[r3,#4*6]			// save system_lr

	// check user stack
	ldr	r0,.cyg_current_thread_table
	ldr	r0,[r0]				// current thread pointer
	ldr	r1,[r0,#THREAD_STACK_BASE]
	ldr	r0,[r0,#THREAD_STACK_SIZE]
	add	r0,r1,r0			// stack top in r0
	add	r1,r1,#512			// stack base + 512 in r1
	cmp	sp,r0				// sp > top ?
	bhi	stack_overflow
	cmp	sp,r1				// sp <= base+512 ?
	bls	stack_overflow

	// check that swi instr address is in domain 0
	mrc	p15,0,r0,c2,c0,0
	mov	r0,r0,lsr #14
	mov	r0,r0,lsl #14			// r0 = ttb base
	mov     r1,r2,lsr #20			// r1 = swi addr >> 20
	mov     r1,r1,lsl #2
	add	r0,r0,r1
	ldr	r0,[r0]				// r0 = level one descriptor
	and	r1,r0,#3			// r1 = descriptor id
	cmp	r1,#0
	beq	invalid_swi			// should not happen
	mov     r1,r0,lsr #5
	and	r1,r1,#0xF			// r1 = domain
	cmp	r1,#0
	bne	invalid_swi
	ldr	ip,[r2]				// ip = system call address

do_call:
	ldmia	r3,{r0-r4}			// restore registers
	mov	lr,pc
	mov	pc,ip				// and do system call

	// we normally come back here, except if exit() was called
return_from_call:
	// switch back to SVC mode with interrupts disabled
	mrs	lr,cpsr
	bic	lr,lr,#CPSR_MODE_BITS
	orr	lr,lr,#CPSR_SUPERVISOR_MODE|CPSR_IRQ_DISABLE|CPSR_FIQ_DISABLE
	msr	cpsr,lr

	ldr	sp,.cyg_current_thread_table
	ldr	sp,[sp]				// current thread pointer
	ldr	lr,.memprot_data_index
	ldr	lr,[lr]                         // lr = memprot data index
	add	sp,sp,#THREAD_DATA	        // sp = per-thread data array
	ldr	sp,[sp,lr,lsl #2]               // sp = user thread data

	// restore DACR
	stmia	sp,{r0-r2}			// save some registers
	ldr	r0,[sp,#4*7]			// get saved DACR
	ldr	r1,=CYGOPT_MEMPROT_DACR_MASK
	and	r0,r0,r1
	mrc	p15,0,r2,c3,c0,0
	bic	r2,r2,r1
	orr	r0,r0,r2
	mcr	p15,0,r0,c3,c0,0		// restore DACR
	ldmia	sp,{r0-r2}

	ldr	lr,[sp,#4*5]			// get svc_spsr
	msr	spsr,lr				// restore spsr
	ldr	lr,[sp,#4*6]			// get system_lr
	movs	pc,lr				// and return from SWI

invalid_swi:
	// simulate a call to cyg_memprot_invalid_swi()
	str	r2,[r3,#0]			// pass faulting address
	ldr	ip,.cyg_memprot_invalid_swi
	bal	do_call

stack_overflow:
	// get a good stack (stack top)
	mov	sp,r0
	// simulate a call to cyg_memprot_stack_overflow()
	ldr	r2,[r3,#4*6]
	str	r2,[r3,#0]			// pass faulting address
	ldr	ip,.cyg_memprot_stack_overflow
	bal	do_call

.cyg_memprot_invalid_swi:
	.word	cyg_memprot_invalid_swi

.cyg_memprot_stack_overflow:
	.word	cyg_memprot_stack_overflow

.cyg_current_thread_table:
	.word	cyg_current_thread_table

.memprot_data_index:
	.word	memprot_data_index
