//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
*******************************************************************************
* @file  ram_alloc.h
* @brief RAM memory allocation.
*
* Memory protection physical RAM allocator. This RAM allocator is designed
* primarily as an ARM MMU physical allocator. It is able to allocate
* contiguous chunks of memory of only 3 possible sizes : 1 MB (section), 64 KB
* (large page) and 4 KB (small page). All allocated chunks are aligned with
* respect to their size. Unlike traditional memory allocators, it is also able
* to partially free an allocated block, provided the requested range to free
* has one of the 3 valid sizes.
*
* Note: This allocator can handle only one bank of RAM memory. It is not
* thread-safe, as it is supposed to be used in a thread-safe context (in MMU
* routines).
*
* Copyright (C) 2004,2005 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2004-06-26
*******************************************************************************
*/
#ifndef CYGONCE_DYNAMO_RAM_ALLOC_H
#define CYGONCE_DYNAMO_RAM_ALLOC_H

#include <cyg/infra/cyg_type.h>

int ram_init( CYG_ADDRESS base, unsigned int *table, int nb_pages );

CYG_ADDRESS ram_allocate(size_t size);

void ram_free( CYG_ADDRESS addr, size_t size );

cyg_uint32 ram_get_used_mem(void);

#endif
