/**
*******************************************************************************
* @file  rtmon_hal.h
* @brief Real-Time Monitor / HAL-dependent routines and constants
*
* Copyright (C) 2006 Parrot S.A.
*
* @author     ivan.djelic@parrot.fr
* @date       2006-12-01
*******************************************************************************
*/
#ifndef CYGONCE_RTMON_HAL_S3C2412_H
#define CYGONCE_RTMON_HAL_S3C2412_H

//-----------------------------------------------------------------------------
// Samsung s3c2412 platform
#include <cyg/hal/s3c2412x.h>

// f = PCLK / 16 
#define TIMER_BASE_FREQUENCY    CYGNUM_HAL_ARM_SMDK2412_PERIPHERAL_CLOCK / 16

//timer period is 10 ms --> 1/100 s then count is f / 100
#define RTMON_TIMER_PERIOD          (TIMER_BASE_FREQUENCY/100)


// timer count for 10ms
#define RTMON_HAL_TICKPERIOD              (RTMON_TIMER_PERIOD)
// System clock has 100/x beats per time unit (100 ns)
#define RTMON_HAL_TIMEFACT_MULT     (1000)
#define RTMON_HAL_TIMEFACT_DIV      (TIMER_BASE_FREQUENCY/(RTMON_HAL_TIMEFACT_MULT*10))
// Set time unit to 100 ns = 10^-7 s
#define RTMON_HAL_TIMESCALE         (-7)


#define SHIFT_TIMER4 14

//<! code to execute during a RTC interrupt
#define RTMON_HAL_LOG_RTC( )

#define RTMON_LOG_STAMP(_type_,_data_) ({                                     \
    cyg_uint32 _r, _pkt;                                                      \
    volatile cyg_uint32 *_base = (volatile cyg_uint32 *)TCFG0;                \
    _pkt = _base[(TCNTO4-TCFG0) >> 2];                                        \
    _base = (volatile cyg_uint32 *)SRCPND;                                    \
    _r = ((_base[0] >> SHIFT_TIMER4 ) & 1) << RTMON_PKT_FLAG_PEND_TICK;       \
    /* re-read value to avoid wrapping issues */                              \
    _base = (volatile cyg_uint32 *)TCFG0;                                     \
    _pkt = ( _r )? _base[(TCNTO4-TCFG0) >> 2] : _pkt;                         \
    _pkt |= (_r|((_data_) << 16)|((_type_) << 24));                           \
    RTMON_LOG(_pkt);                                                          \
    _pkt;                                                                     \
})


#define RTMON_MKTIME32(_pkt) ({                                               \
    cyg_uint32 _corr;                                                         \
    cyg_uint16 _val = (_pkt) & 0xffff;                                        \
    _corr = ((_pkt) >> RTMON_PKT_FLAG_PEND_TICK) & 1;                         \
    (rtmon.ticks+1+_corr)*RTMON_TIMER_PERIOD-_val-1;                       \
})

#define RTMON_MKTIME64(_pkt) ({                                               \
    cyg_uint32 _corr;                                                         \
    cyg_uint16 _val = (_pkt) & 0xffff;                                        \
    _corr = ((_pkt) >> RTMON_PKT_FLAG_PEND_TICK) & 1;                         \
    ((cyg_uint64)(rtmon.ticks+1+_corr))*RTMON_TIMER_PERIOD -_val-1;        \
})

#define RTMON_FIX_PKT(_pkt) ({                                                \
    cyg_uint32 _opkt = _pkt;                                                  \
    (_opkt & 0xffff0000)|((RTMON_TIMER_PERIOD-1)-(_opkt & 0xffff));        \
})

//! Get Link Register
#define RTMON_GET_CALLER() ({                                                 \
    cyg_uint32 _lr;                                                           \
    asm volatile ( "mov %0,lr" : "=r"(_lr));                                  \
    _lr-4;                                                                    \
})

//! Get Program Counter
#define RTMON_GET_PC() ({                                                     \
    cyg_uint32 _pc;                                                           \
    asm volatile ( "mov %0,pc" : "=r"(_pc));                                  \
    _pc-4;                                                                    \
})
/*
#define RTMON_ARM_CHECK_FP(_sbase,_ssize,_x) ({                               \
    cyg_uint32 _cfp = _x;                                                     \
    (( _cfp >= (_sbase))&&                                                    \
     ( _cfp < (_sbase)+(_ssize))&&                                            \
     ( !( _cfp & 0x3 )));                                                     \
})

#define RTMON_GET_CALLER2(_sb, _ss) ({                                        \
    cyg_uint32 _fp, _ret = 0;                                                 \
    asm volatile ("mov %0,fp" : "=r"(_fp));                                   \
    if ( RTMON_ARM_CHECK_FP(_sb,_ss,_fp) ) {                                  \
        _fp = *(volatile cyg_uint32 *)(_fp-12);                               \
        if ( RTMON_ARM_CHECK_FP(_sb,_ss,_fp) ) {                              \
            _ret = (*(volatile cyg_uint32 *)(_fp-4))-4;                       \
        }                                                                     \
    }                                                                         \
    _ret;                                                                     \
})
*/

#define RTMON_HAL_IRQ_TAGS                                  \
    {       [CYGNUM_HAL_INTERRUPT_EINT0]     = "EINT0",     \
            [CYGNUM_HAL_INTERRUPT_EINT1]     = "EINT1",     \
            [CYGNUM_HAL_INTERRUPT_EINT2]     = "EINT2",     \
            [CYGNUM_HAL_INTERRUPT_EINT3]     = "EINT3",     \
            [CYGNUM_HAL_INTERRUPT_EINT4_7]   = "EINT4_7",   \
            [CYGNUM_HAL_INTERRUPT_EINT8_23]  = "EINT8_23",  \
            [CYGNUM_HAL_INTERRUPT_NOTUSED6]  = "NOTUSED6",  \
            [CYGNUM_HAL_INTERRUPT_nBAT_FLT]  = "nBAT_FLT",  \
            [CYGNUM_HAL_INTERRUPT_TICK]      = "TICK",      \
            [CYGNUM_HAL_INTERRUPT_WDT]       = "WDT",       \
            [CYGNUM_HAL_INTERRUPT_TIMER0]    = "TIMER0",    \
            [CYGNUM_HAL_INTERRUPT_TIMER1]    = "TIMER1",    \
            [CYGNUM_HAL_INTERRUPT_TIMER2]    = "TIMER2",    \
            [CYGNUM_HAL_INTERRUPT_TIMER3]    = "TIMER3",    \
            [CYGNUM_HAL_INTERRUPT_TIMER4]    = "TIMER4",    \
            [CYGNUM_HAL_INTERRUPT_UART2]     = "UART2",     \
            [CYGNUM_HAL_INTERRUPT_LCD]       = "LCD",       \
            [CYGNUM_HAL_INTERRUPT_DMA0]      = "DMA0",      \
            [CYGNUM_HAL_INTERRUPT_DMA1]      = "DMA1",      \
            [CYGNUM_HAL_INTERRUPT_DMA2]      = "DMA2",      \
            [CYGNUM_HAL_INTERRUPT_DMA3]      = "DMA3",      \
            [CYGNUM_HAL_INTERRUPT_SDI_CF]    = "SDI_CF",    \
            [CYGNUM_HAL_INTERRUPT_SPI0]      = "SPI0",      \
            [CYGNUM_HAL_INTERRUPT_UART1]     = "UART1",     \
            [CYGNUM_HAL_INTERRUPT_NAND]      = "NAND",      \
            [CYGNUM_HAL_INTERRUPT_USBD]      = "USBD",      \
            [CYGNUM_HAL_INTERRUPT_USBH]      = "USBH",      \
            [CYGNUM_HAL_INTERRUPT_IIC]       = "IIC",       \
            [CYGNUM_HAL_INTERRUPT_UART0]     = "UART0",     \
            [CYGNUM_HAL_INTERRUPT_SPI1]      = "SPI1",      \
            [CYGNUM_HAL_INTERRUPT_RTC]       = "RTC",       \
            [CYGNUM_HAL_INTERRUPT_RTC_ALARM] = "RTC_ALARM", \
            [CYGNUM_HAL_INTERRUPT_ADC]       = "ADC" }


#define RTMON_HAL_US2TIME(_usec) \
    (((_usec)*10*RTMON_HAL_TIMEFACT_DIV)/RTMON_HAL_TIMEFACT_MULT)

#endif /* CYGONCE_RTMON_HAL_S3C2412_H */
