//==========================================================================
// ####ECOSGPLCOPYRIGHTBEGIN####                                            
// -------------------------------------------                              
// This file is part of eCos, the Embedded Configurable Operating System.   
//
// eCos is free software; you can redistribute it and/or modify it under    
// the terms of the GNU General Public License as published by the Free     
// Software Foundation; either version 2 or (at your option) any later      
// version.                                                                 
//
// eCos is distributed in the hope that it will be useful, but WITHOUT      
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License    
// for more details.                                                        
//
// You should have received a copy of the GNU General Public License        
// along with eCos; if not, write to the Free Software Foundation, Inc.,    
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.            
//
// As a special exception, if other files instantiate templates or use      
// macros or inline functions from this file, or you compile this file      
// and link it with other works to produce a work based on this file,       
// this file does not by itself cause the resulting work to be covered by   
// the GNU General Public License. However the source code for this file    
// must still be made available in accordance with section (3) of the GNU   
// General Public License v2.                                               
//
// This exception does not invalidate any other reasons why a work based    
// on this file might be covered by the GNU General Public License.         
// -------------------------------------------                              
// ####ECOSGPLCOPYRIGHTEND####                                              
//==========================================================================

/**
 * @file   rtmon_hal.c
 * @brief  RTmon HAL-dependent routines
 *
 * @author ivan.djelic@parrot.com
 * @date   2008-04-04
 *
 * This file is licensed under the eCos Public License. A copy of this license
 * can be obtained from:
 * http://ecos.sourceware.org/license-overview.html
 *
 * The Original Code is eCos - Embedded Configurable Operating System,
 *
 * Portions created by Parrot S.A. are
 * Copyright (C) 2004-2008 Parrot S.A.
 */

#include <pkgconf/system.h>
#if defined(CYGPKG_HAL_ARM_ARM9_P5DEV)

#include <cyg/kernel/kapi.h>
#include <cyg/infra/diag.h>         // diagnostic output
#include <pkgconf/rtmon.h>
#include <cyg/rtmon/rtmon.h>
#include <cyg/rtmon/rtmon_common.h>
#include <cyg/rtmon/rtmon_internal.h>
#include <cyg/rtmon/rtmon_hal.h>
#include <cyg/crc/crc.h>
#include <cyg/hal/hal_diag.h>

//! RTmon profiling period 50s
#define RTMON_PROFPERIOD_50US 0x146

//-----------------------------------------------------------------------------
// Parrot5/5+ platform

void rtmon_hal_start_timer(void)
{
    /* do nothing, we are using the system timer */
}

void rtmon_hal_init_var(void)
{
    /* Nothing to init */
}

void rtmon_hal_stop_timer(void)
{
    /* do nothing, we are using the system timer */
}

//! declaration of the swi_handler routine from swi_handler.S
cyg_VSR_t swi_rtmon_handler;


void rtmon_hal_start_profiling(void)
{
    /* allocate timer 2 */
    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_TIM2CTL, 0);
    HAL_WRITE_UINT32((PARROT5_VIC+_VIC_IntSelect),
                     1 << CYGNUM_HAL_INTERRUPT_TIMER2);
    cyg_interrupt_set_vsr(CYGNUM_HAL_VECTOR_FIQ, swi_rtmon_handler);
    cyg_drv_interrupt_unmask(CYGNUM_HAL_INTERRUPT_TIMER2);

    /* restart timer */
    HAL_WRITE_UINT32((PARROT5_SYS+_SYS_TIM2LD ),
                     RTMON_PROFPERIOD_50US );
    HAL_WRITE_UINT32((PARROT5_SYS+_SYS_TIM2CTL),
                     SYS_TIMXCTL_PERIODIC  |
                     SYS_TIMXCTL_PRESCALE16|
                     SYS_TIMXCTL_ENABLE);
}

void rtmon_hal_stop_profiling(void)
{
    // disable Timer 2
    cyg_drv_interrupt_mask(CYGNUM_HAL_INTERRUPT_TIMER2);
    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_TIM2CTL, 0);
    // clear interrupt flag
    HAL_WRITE_UINT32(PARROT5_SYS+_SYS_FLAGS, 1 << 3);
}

#endif //CYGPKG_HAL_ARM_ARM9_P5DEV
